///////////////////////////////////////////////////////////////////////////////
//
//                             A  C  S  N
//
// +---------------  Alaska Certified Solutions Network  -------------------+
// |                                                                        |
// |        This file is proved and certified by Alaska Software            |
// |                                                                        |
// |                   No: <Certification number>                           |
// |                          109xxx-04-0014                                |
// |                                                                        |
// |   For more information about ACSN read the appropriate announcement    |
// |      or scan for ACSN in the Alaska Support-LIBs on CompuServe or      |
// |                   at WWW.ALASKA-SOFTWARE.COM                           |
// |                                                                        |
// +------------------------------------------------------------------------+
//
// FILE NAME
//
//    Options.prg
//
// AUTHOR
//
//    (c) Copyright 1999, Hannes Ziegler
//
//    ALL RIGHTS RESERVED
//
//    This file is the property of AUTHOR. It participates in the
//    Alaska Certified Solutions Network program. Permission to use,
//    copy, modify, and distribute this software for any purpose and
//    without fee is hereby granted, provided that the above copyright
//    notice appear in all copies and that the name of the author or
//    Alaska Software not be used in advertising or publicity pertaining
//    to distribution of the software without specific, written prior
//    permission.
//
// WARRANTY
//
//    THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
//    AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
//    INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
//    FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR
//    OR ALASKA SOFTWARE BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
//    SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
//    INCLUDING WITHOUT LIMITATION, LOSS OF PROFIT AND LOSS OF USE.
//
// DESCRIPTION
//
//    This file implements the Options window for the Event Spy.
//
// REMARKS
//
//    See EVENTSPY.PRG for full information.
//
// HISTORY
//
//    01/31/99 Creation of file
//
///////////////////////////////////////////////////////////////////////////////

#include "Gra.ch"
#include "Xbp.ch"
#include "Font.ch"
#include "Common.ch"
#include "Appevent.ch"

#define SPY_OPTIONS_FILE   "OPTIONS.SPY"   // Binary configuration data file

CLASS EventSpyOptionsDialog FROM XbpDialog
   PROTECTED:
   VAR aHotKeys

   EXPORTED:
   VAR tabSettings   
   VAR checkOnTop    
   VAR checkAddHidden
   VAR checkAutoSize
   VAR checkFixedFont

   VAR tabInfo
   VAR checkFocus
   VAR checkThreadID 
   VAR checkTimeStamp
   VAR checkShort
   VAR checkMp2

   VAR tabFilter     
   VAR checkToSpy    
   VAR checkPaint    
   VAR checkMotion   

   VAR tabOptions    
   VAR Static1       
   VAR comboHotKey   
   VAR Static2      
   VAR spinMaxEvents 
   VAR buttonRead    

   VAR ButtonDefault      
   VAR ButtonSave  

   VAR currentTab

   METHOD init
   METHOD create, show
   METHOD getData, setData
ENDCLASS

******************************************************************************
* Initialize form
******************************************************************************
METHOD EventSpyOptionsDialog:init( oParent, oOwner, aPos, aSize, aPP, lVisible )

   DEFAULT oParent  TO AppDesktop()           , ;
           oOwner   TO EventSpy()             , ;
           aPos     TO EventSpy():currentpos(), ;
           aSize    TO {276,265}              , ;
           lVisible TO .F.                    , ;
           aPP      TO {}

   AAdd ( aPP, { XBP_PP_BGCLR, GRA_CLR_PALEGRAY } )
   AAdd ( aPP, { XBP_PP_COMPOUNDNAME, FONT_HELV_SMALL } )

   ::XbpDialog:init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   ::XbpDialog:drawingArea:ColorBG := GRA_CLR_PALEGRAY  
   ::XbpDialog:border    := XBPDLG_RECESSEDBORDERTHICK_FIXED
   ::XbpDialog:taskList  := .F.
   ::XbpDialog:title     := "Event Spy Configuration"
   ::XbpDialog:close     := {|mp1,mp2,obj| obj:hide() }
   ::XbpDialog:moveWithOwner := .F.

   ::tabSettings    := XbpTabPage():new( ::drawingArea, , {4,48}, {260,176} )
   ::tabSettings:colorBG := GRA_CLR_PALEGRAY
   ::tabSettings:caption := "Settings"
   ::tabSettings:clipSiblings := .T.
   ::tabSettings:minimized    := .F.
   ::tabSettings:preOffset    :=  0
   ::tabSettings:postOffset   := 75
   ::currentTab := ::tabSettings

   ::checkOnTop     := XbpCheckbox():new( ::tabSettings, , {32,112}, {205,24} )
   ::checkOnTop:caption := "stay on top"
   ::checkOnTop:clipSiblings := .T.

   ::checkAddHidden := XbpCheckbox():new( ::tabSettings, , {32,88}, {205,24} )
   ::checkAddHidden:caption := "add events when hidden"
   ::checkAddHidden:clipSiblings := .T.

   ::checkAutoSize  := XbpCheckbox():new( ::tabSettings, , {32,64}, {205,24} )
   ::checkAutoSize:caption := "auto-adjust to Log width"
   ::checkAutoSize:clipSiblings := .T.

   ::checkFixedFont := XbpCheckbox():new( ::tabSettings, , {32,40}, {205,24} )
   ::checkFixedFont:caption := "use fixed font"
   ::checkFixedFont:clipSiblings := .T.


   ::tabInfo      := XbpTabPage():new( ::drawingArea, , {4,48}, {260,176} )
   ::tabInfo:colorBG := GRA_CLR_PALEGRAY
   ::tabInfo:caption := "Log Info"
   ::tabInfo:clipSiblings := .T.
   ::tabInfo:minimized    := .T.
   ::tabInfo:preOffset    := 25
   ::tabInfo:postOffset   := 50

   ::checkFocus     := XbpCheckbox():new( ::tabInfo, , {32,112}, {205,24} )
   ::checkFocus:caption := "mark XBP with input focus"
   ::checkFocus:clipSiblings := .T.

   ::checkThreadID  := XbpCheckbox():new( ::tabInfo, , {32,88}, {205,24} )
   ::checkThreadID:caption := "display thread ID"
   ::checkThreadID:clipSiblings := .T.

   ::checkTimeStamp := XbpCheckbox():new( ::tabInfo, , {32,64}, {205,24} )
   ::checkTimeStamp:caption := "display time stamp"
   ::checkTimeStamp:clipSiblings := .T.

   ::checkShort     := XbpCheckbox():new( ::tabInfo, , {32,40}, {205,24} )
   ::checkShort:caption := "display short event constants"
   ::checkShort:clipSiblings := .T.

   ::checkMp2       := XbpCheckbox():new( ::tabInfo, , {32,16}, {205,24} )
   ::checkMp2:caption := "display 2nd message parameter"
   ::checkMp2:clipSiblings := .T.


   ::tabFilter      := XbpTabPage():new( ::drawingArea, , {4,48}, {260,176} )
   ::tabFilter:colorBG := GRA_CLR_PALEGRAY
   ::tabFilter:caption := "Filter"
   ::tabFilter:clipSiblings := .T.
   ::tabFilter:minimized    := .T.
   ::tabFilter:preOffset    := 50
   ::tabFilter:postOffset   := 25

   ::checkToSpy     := XbpCheckbox():new( ::tabFilter, , {32,112}, {205,24} )
   ::checkToSpy:caption := "show events for Spy window"
   ::checkToSpy:clipSiblings := .T.

   ::checkPaint     := XbpCheckbox():new( ::tabFilter, , {32,88}, {205,24} )
   ::checkPaint:caption := "show Paint events"
   ::checkPaint:clipSiblings := .T.

   ::checkMotion    := XbpCheckbox():new( ::tabFilter, , {32,64}, {205,24} )
   ::checkMotion:caption := "show Motion events"
   ::checkMotion:clipSiblings := .T.


   ::tabOptions     := XbpTabPage():new( ::drawingArea, , {4,48}, {260,176} )
   ::tabOptions:colorBG := GRA_CLR_PALEGRAY
   ::tabOptions:caption := "Options"
   ::tabOptions:clipSiblings := .T.
   ::tabOptions:minimized    := .T.
   ::tabOptions:preOffset    := 75
   ::tabOptions:postOffset   :=  0

   ::Static1         := XbpStatic():new( ::tabOptions, , {24,108}, {64,24} )
   ::Static1:caption := "Hot key"
   ::Static1:clipSiblings := .T.
   ::Static1:options := XBPSTATIC_TEXT_VCENTER+XBPSTATIC_TEXT_RIGHT

   ::comboHotKey    := XbpCombobox():new( ::tabOptions, , {96,48}, {96,84} )
   ::comboHotKey:clipSiblings := .T.
   ::comboHotKey:type := XBPCOMBO_DROPDOWNLIST

   ::Static2         := XbpStatic():new( ::tabOptions, , {14,72}, {74,20} )
   ::Static2:caption := "Max. events"
   ::Static2:clipSiblings := .T.
   ::Static2:options := XBPSTATIC_TEXT_VCENTER+XBPSTATIC_TEXT_RIGHT

   ::spinMaxEvents  := XbpSpinbutton():new( ::tabOptions, , {96,72}, {96,24} )
   ::spinMaxEvents:clipSiblings := .T.
   

   ::buttonRead     := XbpPushButton():new( ::tabOptions, , {16,20}, {176,28} )
   ::buttonRead:caption := "Read AppEvent.ch again"
   ::buttonRead:clipSiblings := .T.
   ::buttonRead:activate := {|| EventSpy():parseAppEventCH() }

   ::ButtonSave   := XbpPushButton():new( ::drawingArea, , {12,8}, {80,24} )
   ::ButtonSave:caption := "Save"
   ::ButtonSave:clipSiblings := .T.
   ::ButtonSave:activate := {|| ::getData( .T. ), ::hide() }

   ::ButtonDefault   := XbpPushButton():new( ::drawingArea, , {128,8}, {80,24} )
   ::ButtonDefault:caption := "Default"
   ::ButtonDefault:clipSiblings := .T.
   ::ButtonDefault:activate := {|| EventSpy():options:setDefault(), ::setData() }

   ::aHotKeys := {"F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12"}


   // Code block to minimize/maximize Tab pages
   ::tabSettings:tabActivate := ;
   ::tabInfo    :tabActivate := ;
   ::tabFilter  :tabActivate := ;
   ::tabOptions :tabActivate := ;
      {|mp1,mp2,obj| ::currentTab:minimize(), obj:maximize(), ::currentTab := obj }

   // I'm lazy here: assign _all_ options if _one_ option is changed
   ::checkOnTop    :selected := ;
   ::checkAddHidden:selected := ;
   ::checkAutoSize :selected := ;
   ::checkFixedFont:selected := ;
   ::checkFocus    :selected := ;
   ::checkThreadID :selected := ;
   ::checkTimeStamp:selected := ;
   ::checkShort    :selected := ;
   ::checkMp2      :selected := ;
   ::checkToSpy    :selected := ;
   ::checkPaint    :selected := ;
   ::checkMotion   :selected := ;
   ::spinMaxEvents :endSpin  := {|| ::getData() }
RETURN self


******************************************************************************
* Request system resources
******************************************************************************
METHOD EventSpyOptionsDialog:create( oParent, oOwner, aPos, aSize, aPP, lVisible )
   
   ::XbpDialog:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

   ::tabSettings   :create()
   ::checkOnTop    :create()
   ::checkAddHidden:create()
   ::checkAutoSize :create()
   ::checkFixedFont:create()

   ::tabInfo       :create()
   ::checkFocus    :create()
   ::checkThreadID :create()
   ::checkTimeStamp:create()
   ::checkShort    :create()
   ::checkMp2      :create()

   ::tabFilter     :create()
   ::checkToSpy    :create()
   ::checkPaint    :create()
   ::checkMotion   :create()

   ::tabOptions    :create()
   ::Static1       :create()
   ::comboHotKey   :create()
   ::Static2       :create()
   ::spinMaxEvents :create()
   ::spinMaxEvents :setNumLimits( 10, 1000 )
   ::buttonRead    :create()
   ::ButtonDefault :create()
   ::ButtonSave    :create()

   AEval( ::aHotKeys, {|c| ::comboHotKey:addItem( c ) } )

RETURN self


METHOD EventSpyOptionsDialog:show
   ::setData()
   ::XbpDialog:show()
RETURN


METHOD EventSpyOptionsDialog:setData()
   LOCAL oOptions := EventSpy():options
   LOCAL cHotKey  := EventSpy():translate( oOptions:hotKey )

   CutStr( "xbeK_", @cHotKey )

   ::checkOnTop    :setData( oOptions:stayOnTop     )
   ::checkAddHidden:setData( oOptions:addHidden     )
   ::checkAutoSize :setData( oOptions:autoSize      )
   ::checkFixedFont:setData( oOptions:useFixedFont  )

   ::checkFocus    :setData( oOptions:showFocus     )
   ::checkThreadID :setData( oOptions:showThreadID  )
   ::checkTimeStamp:setData( oOptions:showTimeStamp )
   ::checkShort    :setData( oOptions:showShort     )
   ::checkMp2      :setData( oOptions:showMp2       )

   ::checkToSpy    :setData( oOptions:showToSpy     )
   ::checkPaint    :setData( oOptions:showPaint     )
   ::checkMotion   :setData( oOptions:showMotion    )
   ::spinMaxEvents :setData( oOptions:maxEvents     )

   ::comboHotKey:xbpSLE:setData( cHotKey )
RETURN self


METHOD EventSpyOptionsDialog:getData()
   LOCAL oOptions := EventSpy():options
   LOCAL cHotKey  := "xbeK_" + ::comboHotKey:xbpSLE:getData()
   LOCAL nHotKey  := EventSpy():translate( cHotKey )

   oOptions:stayOnTop     := ::checkOnTop    :getData()
   oOptions:addHidden     := ::checkAddHidden:getData()
   oOptions:autoSize      := ::checkAutoSize :getData()
   oOptions:useFixedFont  := ::checkFixedFont:getData()

   oOptions:showFocus     := ::checkFocus    :getData()
   oOptions:showThreadID  := ::checkThreadID :getData()
   oOptions:showTimeStamp := ::checkTimeStamp:getData()
   oOptions:showShort     := ::checkShort    :getData()
   oOptions:showMp2       := ::checkMp2      :getData()

   oOptions:showToSpy     := ::checkToSpy    :getData()
   oOptions:showPaint     := ::checkPaint    :getData()
   oOptions:showMotion    := ::checkMotion   :getData()

   oOptions:maxEvents     := ::spinMaxEvents :getData()
   oOptions:hotKey        := nHotKey

   EventSpy():updateOptions()

   IF PCount() > 0
      // :save() is a CLASS METHOD and oOptions is an instance! Therefore,
      // the instance must be passed as parameter to the class method
      oOptions:save( oOptions )
   ENDIF
RETURN self


******************************************************************************
* Class which holds the configuration values of the EventSpy and 
* saves/restores these values
*
* Note: an Options object is made persistent 
******************************************************************************
CLASS EventSpyOptions
   EXPORTED:
   VAR pos                             // state of EventSpy window
   VAR size                            // at program termination
   VAR frameState                      //

   VAR stayOnTop                       // TRUE/FALSE options
   VAR addHidden                       //
   VAR autoSize                        //
   VAR useFixedFont                    //
   VAR showFocus                       //
   VAR showThreadID                    //
   VAR showTimeStamp                   //
   VAR showShort                       //
   VAR showMp2                         //
   VAR showToSpy                       //
   VAR showPaint                       //
   VAR showMotion                      //

   VAR hotKey                          // hotkey to recall minimized window
   VAR maxEvents                       // max. # of log entries

   CLASS VAR includePath               // path for APPEVENT.CH
   CLASS VAR libPath                   // path for EVENTSPY.DLL

   CLASS METHOD initClass
   CLASS METHOD save, restore

   METHOD init
   METHOD setDefault
ENDCLASS


CLASS METHOD EventspyOptions:initClass()
   ::includePath := GetEnvDir( "INCLUDE", "APPEVENT.CH"  )
   ::libPath     := GetEnvDir( "LIB"    , "EVENTSPY.DLL" )
RETURN self


CLASS METHOD EventSpyOptions:save( oOptions )

   // save pos/size only if Event Spy is neither minimized nor maximized
   IF EventSpy():getFrameState() == XBPDLG_FRAMESTAT_NORMALIZED
      oOptions:pos     := EventSpy():currentPos()
      oOptions:size    := EventSpy():currentSize()
   ENDIF

   oOptions:frameState := EventSpy():getFrameState()
RETURN WriteStream( ::libPath + SPY_OPTIONS_FILE, Var2Bin(oOptions) )


CLASS METHOD EventspyOptions:restore()
   LOCAL oOptions, cBinary, cFile := ::libPath + SPY_OPTIONS_FILE

   #pragma Info( NOINIT )              // suppress compiler warnings
                                       // for cBinary
   IF File( cFile ) .AND. ReadStream( cFile, @cBinary )
      oOptions := Bin2Var( cBinary )
   ELSE
      oOptions := ::new()
   ENDIF
   #pragma Info( INIT )
RETURN oOptions


METHOD EventspyOptions:init()
   ::pos           := NIL              // these values are assigned
   ::size          := NIL              // by the EventSpy
   ::frameState    := XBPDLG_FRAMESTAT_NORMALIZED
   ::hotKey        := xbeK_F12
   ::maxEvents     := 100
   ::setDefault()
RETURN self


METHOD EventspyOptions:setDefault()
   ::stayOnTop     := .F.              // default values for
   ::addHidden     := .F.              // Spy options.  The method
   ::autoSize      := .F.              // is convenient for resetting 
   ::useFixedFont  := .F.              // all checkboxes at once.

   ::showFocus     := .F.              // Edit these values to suit
   ::showThreadID  := .F.              // your favorite default settings
   ::showTimeStamp := .F.
   ::showshort     := .F.
   ::showMp2       := .T.

   ::showToSpy     := .F.
   ::showPaint     := .F.
   ::showMotion    := .F.
RETURN self

//EOF
/////