///////////////////////////////////////////////////////////////////////////////
//
//                             A  C  S  N
//
// +---------------  Alaska Certified Solutions Network  -------------------+
// |                                                                        |
// |        This file is proved and certified by Alaska Software            |
// |                                                                        |
// |                   No: <Certification number>                           |
// |                          109xxx-04-0014                                |
// |                                                                        |
// |   For more information about ACSN read the appropriate announcement    |
// |      or scan for ACSN in the Alaska Support-LIBs on CompuServe or      |
// |                   at WWW.ALASKA-SOFTWARE.COM                           |
// |                                                                        |
// +------------------------------------------------------------------------+
//
// FILE NAME
//
//    Testspy.prg
//
// AUTHOR
//
//    (c) Copyright 1998, Hannes Ziegler
//
//    ALL RIGHTS RESERVED
//
//    This file is the property of AUTHOR. It participates in the
//    Alaska Certified Solutions Network program. Permission to use,
//    copy, modify, and distribute this software for any purpose and
//    without fee is hereby granted, provided that the above copyright
//    notice appear in all copies and that the name of the author or
//    Alaska Software not be used in advertising or publicity pertaining
//    to distribution of the software without specific, written prior
//    permission.
//
// WARRANTY
//
//    THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
//    AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
//    INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
//    FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR
//    OR ALASKA SOFTWARE BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
//    SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
//    INCLUDING WITHOUT LIMITATION, LOSS OF PROFIT AND LOSS OF USE.
//
// DESCRIPTION
//
//    This file demonstrates the usage of the Event Spy. I found it
//    convenient to have the Event Spy available in all programs with
//    the least possible efforts. Therefore, I decided to create the
//    Event Spy in a DLL that is statically linked to the EXE.
//
//    Copy EVENTSPY.DLL and EVENTSPY.LIB to a directory of the LIB path.
//    (e.g: \ALASKA\XPPW32\LIB) and link EVENTSPY.LIB to your application
//
// REMARKS
//
//    For more information see the ACSN header in EVENTSPY.PRG
//
// HISTORY
//
//    02/12/99 Added code to demonstrate how to hook into the event log
//             and removed code that became obsolete with the new version
//
//    08/08/98 Removed the obsolete commands INIT EVENTSPY and SPY
//    07/02/98 Creation of file
//
///////////////////////////////////////////////////////////////////////////////

#include "Appevent.ch"
#include "Appedit.ch"
#include "Common.ch"
#include "Gra.ch"
#include "Xbp.ch"

******************************************************************************
* Main procedure to test the Event Spy
******************************************************************************

PROCEDURE Main
   LOCAL nEvent, oXbp, mp1, mp2

   // If there is a DBF in the current directory, display an
   // APPEDIT window to serve as receiver for events
   mp1 := Directory( "*.DBF" )
   IF ! Empty( mp1 )
      #pragma Info( NONE )
      USE (mp1[1,1])
      APPEDIT POSITION TOP, CENTER
      APPDISPLAY MODELESS
      #pragma Info( ALL )
   ENDIF

   // All events are be created for SetAppWindow() (-> XbpCrt Window)
   SetMouse( .T. )
   SetAppWindow():useShortCuts := .T.
   SetAppWindow():close        := {|| AppQuit() }

   ? "This XbpCrt Window has been created in the default AppSys() procedure"
   ?
   ? "Open the 'Options Window' and Select all three checkboxes in the'"
   ? "Filter tab page for initial testing"
   ?
   ? '  [x]  "show events for Spy window"'
   ? '  [x]  "show Motion events"'
   ? '  [x]  "show Paint events"'
   ?
   ? "Then click around or use the keyboard to see what happens"
   ?
   ? "You may also double click an entry in the event Log"

   MsgBox( "The Event Spy is visible now" +Chr(13) + ;
           "although it is not yet invoked in the test program", "NOTE" )

   SetAppFocus( SetAppWindow() )
   
   // This demonstrates how to hook into the event log
   EventSpy():logSelected := {|a| MyFunc( a ) }

   DO WHILE .T.
      #pragma Info( NOINIT )
      nEvent := AppEvent( @mp1, @mp2, @oXbp, 1500 )
      #pragma Info( INIT )

      IF oXbp <> NIL
         oXbp:HandleEvent( nEvent, mp1, mp2 )
      ENDIF

      IF nEvent == xbeP_Quit
         QUIT
      ENDIF
   ENDDO

RETURN



FUNCTION MyFunc( aEvent )
   LOCAL cString

   cString := EventSpy():format( aEvent[1], aEvent[2], aEvent[3], aEvent[4] )

   IF aEvent[1] <> NIL
      MsgBox( "The numeric event constant is:" + Chr(13) +Chr(13) + ;
               LTrim( Str(aEvent[1]) ), "Hooked into the Event Spy" ) 
   ENDIF

   QOut( cString )

RETURN cString


PROCEDURE AppQuit
   QUIT
RETURN
