// Using ASINET to create a Client/Server application

/*
CLIENT.PRG is an Xbase++ application that is used to poll a server
for data.  The default server is running on DONNAY-SOFTWARE.COM.
This server is an .EXE created from SERVER.PRG.  The client program
tells the server what to do as a set of instruction code blocks, and
the server returns the results in an array.

Send an email to rogerdonnay@donnay-software.com for login instructions.

IF you cannot get a connection to DONNAY-SOFTWARE.COM, start up
SERVER.EXE on your computer then start up CLIENT.EXE as follows:

CLIENT LOCALHOST

*/


#include "Asinetc.ch"
#include "Inkey.ch"
#INCLUDE "dcdialog.ch"
#INCLUDE "appevent.ch"

#pragma library( "ASINet1c.lib" )

PROCEDURE DbeSys
   // Don't load a Database Engine
   // Data stored in a database is retrieved from the server process
RETURN

PROCEDURE AppSys
RETURN

PROCEDURE Main( cServer )

   LOCAL xCustData, bRetrieve, oBrowse, GetList := {}, aFieldNames, aData, ;
         GetOptions, nElement, oMsgBox, cCommand, oStatic, nPointer

   DEFAULT cServer := "donnay-software.com"

   PUBLIC pnPort := 1042, ;
          pcServer := cServer, ;
          pcUserId := Space(20), ;
          pcPassword := Space(10)

   aData := {}
   bRetrieve := ;
      {|nMode|Retrieve(cCommand,oBrowse,aData,nPointer,aFieldNames, ;
                       oMsgBox,nMode)}

   cCommand := 'DC_ReadGui'

   @ 1,0 DCSAY 'User Id' GET M->pcUserID
   @ 2,0 DCSAY 'Password' GET M->pcPassword // PASSWORD

   @ 4,0 DCSAY "Enter Command/Function" GET cCommand PICT '@!' ;
         SAYRIGHTBOTTOM ;
         KEYBLOCK {|a,b,o|IIF(a==xbeK_ENTER, ;
               (o:getData(),Eval(bRetrieve,1)),nil)}

   @ 4,45 DCSAY 'Double-click item in browse window to retrieve memos' ;
          SAYLEFTBOTTOM COLOR GRA_CLR_DARKPINK SAYSIZE 0

   @ 6,0 DCBROWSE oBrowse SIZE 100,15 DATA aData PARENT oStatic ;
         PRESENTATION DC_BrowPres() FONT '8.Courier' FIT NOHSCROLL ;
         ITEMSELECTED {||Eval(bRetrieve,2)} ;
         POINTER nPointer

   nElement := 1
   DCSETPARENT TO oBrowse
   DCBROWSECOL ELEMENT nElement++ HEADER 'Command' WIDTH 20
   DCBROWSECOL ELEMENT nElement++ HEADER 'Type' WIDTH 15
   DCBROWSECOL ELEMENT nElement++ HEADER 'Short' WIDTH 60
   DCSETPARENT TO

   aFieldNames := { 'COMMAND', 'TYPE', 'SHORT', 'RecNo()' }

   @ 21.5,0 DCPUSHBUTTON CAPTION 'Retrieve' SIZE 10,1.2 ;
          ACTION {||Eval(bRetrieve,1)}

   @ 21.2,20 DCSAY Version() ;
           COLOR GRA_CLR_BLUE FONT '12.Courier Bold' ;
           SAYSIZE 0

   @ 23,0 DCSAY '' OBJECT oMsgBox SIZE 50 ;
          COLOR GRA_CLR_BLUE

   DCGETOPTIONS ;
      AUTORESIZE ;
      SAYRIGHT ;
      SAYWIDTH 150 ;
      TABSTOP

   DCREAD GUI FIT MODAL OPTIONS GetOptions ;
      TITLE 'eXPress++ Online Documentation' ;
      SETAPPWINDOW

RETURN

* --------

FUNCTION Retrieve( cCommand, oBrowse, aData, nPointer, aFieldNames, ;
                   oMsgBox, nMode )

LOCAL xData, i, nRecord, aInstructions

IF nMode == 1

  ASize(aData,0)
  oBrowse:refreshAll()
  aInstructions := { ;
    { .f., {|| Set(_SET_DEFAULT,'\EXP17\XDOC')} }, ;
    { .f., {|| dbUseArea(.t.,'FOXCDX','express') } }, ;
    { .f., {|| OrdListAdd('express.cdx')} }, ;
    { .f., {|| OrdSetFocus('COMMAND')} }, ;
    { .f., {|| DC_SetScope(0,Trim(cCommand))} }, ;
    { .f., {|| DC_SetScope(1,Trim(cCommand))} }, ;
    { .t., {||M->a:={}, DC_DbEval({||AAdd(M->a,Scatter(,aFieldNames))}),M->a} }, ;
    { .f., {|| dbCloseAll() } } }

ELSEIF nMode == 2

  nRecord := aData[nPointer,4]

  aInstructions := { ;
    { .F., {|| Set(_SET_DEFAULT,'\EXP17\XDOC')} }, ;
    { .f., {|| dbUseArea(.t.,'FOXCDX','express') } }, ;
    { .f., {|| dbGoto(nRecord)} }, ;
    { .t., {|| {EXPRESS->descript, EXPRESS->syntax, EXPRESS->args} } }, ;
    { .f., {|| dbCloseAll() } } }

ENDIF

xData := GetData( oMsgBox, aInstructions )

IF nMode = 1 .AND. Valtype( xData ) == "A"
  FOR i := 1 TO Len(xData)
    AAdd(aData,xData[i])
  NEXT i
  oBrowse:refreshAll()
ELSEIF nMode = 2 .AND. Valtype(xData) == 'A'
  DisplayMemos(xData,aData[nPointer,1])
ELSEIF Valtype( xData ) = 'C'
  DC_WinAlert(xData)
ENDIF

RETURN nil

/*
 * Retrieve customer data from a database managed by a server process
 */
FUNCTION GetData( oMsgBox, aInstructions )

   LOCAL nSocket
   LOCAL aInstruct
   LOCAL cBuffer
   LOCAL cLength
   LOCAL nBytes
   LOCAL nError := NIL
   LOCAL nPort := 1042
   LOCAL i, nLength := 1, cBuff, nHandle
   LOCAL nTotal := 0, nByte

   /*
    * Create a bound socket to communicate with the server process
    */
   nSocket := SocketOpen( SOCK_STREAM, M->pcServer, M->pnPort, @nError )

   IF nError <> 0
      RETURN "ERROR:" + LTrim(DC_XtoC(nError))
   ENDIF

   /*
    * Convert the array of code blocks to binary data and
    * obtain length of binary string
    */
   cBuffer   := Var2Bin( { { Alltrim(M->pcUserId), Alltrim(M->pcPassword) }, aInstructions  } )
   cLength   := U2Bin( Len( cBuffer ) )

   oMsgBox:SetCaption( "Sending request to: "+ M->pcServer )

   FOR i := 1 to Len( cBuffer ) Step 1000
     cBuff := Substr( cBuffer, i, 1000  )
     cLength := U2Bin( Len( cBuff ) )
     SocketSend( nSocket, cLength + cBuff )
     cBuff := ' '
     SocketRecv( nSocket, @cBuff, 1, ,@nError )
     IF cBuff <> Chr(6) .OR. nError <> 0
       oMsgBox:SetCaption( "ERROR Sending data" )
       EXIT
     ENDIF
   NEXT

   IF nError == 0
     cLength := U2Bin( 0 )
     SocketSend( nSocket, cLength  )
   ENDIF

   cBuffer := ''

   DO WHILE nLength > 0 .AND. nError == 0
     nBytes := SocketRecv( nSocket, @cLength, 4, ,@nError )

     IF nError <> 0 .OR. nBytes <> 4
       SocketClose( nSocket )
       cBuffer := "ERROR:" + LTrim(Str(nError)) + " BYTES RECEIVED:" + LTrim(Str(nBytes))
       RETURN cBuffer
     ENDIF
     // Retrieve binary data created in the server process from socket
     nBytes  := Bin2U( cLength )
     nTotal += nBytes
     oMsgBox:SetCaption( "Receiving "+DC_XtoC(nTotal) +" bytes from server.")

     IF ( nLength := nBytes ) > 0  .and. nBytes <= 32000
       cBuff := Space( nBytes )
       nByte  := SocketRecv( nSocket, @cBuff, nBytes, ,@nError )
       cBuffer += cBuff
       IF nByte == nBytes .and. nError == 0
         SocketSend( nSocket, Chr(6) ) //ACK
       ELSE
         SocketSend( nSocket, Chr(21) )  //NAK
         oMsgBox:SetCaption( "Error in data transfer.")
         EXIT
       ENDIF
     ELSEIF nBytes > 32000
       oMsgBox:SetCaption( "Error in data transfer.")
       EXIT
     ENDIF
   ENDDO

   SocketClose( nSocket )

   IF nError <> 0
     cBuffer := "ERROR:" + LTrim(Str(nError)) + " BYTES RECEIVED:" + LTrim(Str(nBytes))
     RETURN cBuffer
   ENDIF

  // debugging
  nHandle := FCreate('CLIENT.VAR')
  FWrite(nHandle,cBuffer)
  FClose(nHandle)

// Return the result created in the server process to calling function

RETURN Bin2Var( cBuffer )

* ----------------

FUNCTION DisplayMemos( aMemos, cTitle )

LOCAL GetList[0], aTab[3]

@ 0,0 DCTABPAGE aTab[1] SIZE 80,20 CAPTION 'Desc'
@ 0,0 DCTABPAGE aTab[2] RELATIVE aTab[1] CAPTION 'Syntax'
@ 0,0 DCTABPAGE aTab[3] RELATIVE aTab[2] CAPTION 'Args'

@ 2,2 DCMULTILINE aMemos[1] SIZE 76,17 FONT '8.Courier' PARENT aTab[1]
@ 2,2 DCMULTILINE aMemos[2] SIZE 76,17 FONT '8.Courier' PARENT aTab[2]
@ 2,2 DCMULTILINE aMemos[3] SIZE 76,17 FONT '8.Courier' PARENT aTab[3]

DCREAD GUI FIT BUTTONS DCGUI_BUTTON_EXIT MODAL ;
       TITLE cTitle

RETURN nil



