#include "Appevent.ch"
#include "Asinetc.ch"
#include 'dcdialog.ch'

#pragma library( "ASINet1c.lib" )

REQUEST DC_DebugQout


/*
 * Main() handles user input for server termination
 */
PROCEDURE Main( cPort )

   LOCAL nEvent, mp1, mp2, oXbp, oThread

   PUBLIC pnPort := 1042

   IF PCount()>=1 .AND. Val(cPort) > 0
     pnPort := Val(cPort)
   ENDIF

   CLS
   ? 'Using Port ' + Alltrim(Str(pnPort))
   ? 'Version: ' + Version()

   LoadDbes()
   dbeSetDefault('FOXCDX')

   oThread := ServerThread():new( pnPort )
   oThread:start()

   IF .NOT. oThread:active
      ? "Server start-up failed"
      QUIT
   ELSE
      ? "Server up and running ..."
      ? "Press ESC to QUIT"
   ENDIF

   DO WHILE nEvent<> xbeP_Close
      nEvent := AppEvent( @mp1, @mp2, @oXbp )
      oXbp:handleEvent( nEvent, mp1, mp2 )

      IF nEvent == xbeK_ESC
         EXIT
      ENDIF
   ENDDO

   oThread:stop()
RETURN


/* ***************************************************************************
 *
 * This class listens on a socket for client connections
 */
CLASS ServerThread FROM Thread
   PROTECTED:
   VAR nPort
   VAR nSocket

   EXPORTED:
   METHOD init, atStart, execute, atEnd, stop
   METHOD logClient
ENDCLASS


METHOD ServerThread:init( pnPort )
   ::Thread:init()
   ::nSocket := 0
   ::nPort   := pnPort
RETURN self


/*
 * Thread starts!
 *
 * Create socket, put it into listening state and set time interval
 * to zero, so that :execute() is repeated automatically
 */
METHOD ServerThread:atStart
   ::nSocket := SocketCreate( SOCK_STREAM, ::nPort )

   IF ::nSocket <> 0
      SocketListen( ::nSocket )
      ::setInterval(0)
   ENDIF
RETURN self


/*
 * Thread is running!
 *
 * Accept a client connection and spawn new thread for data transmission
 */
METHOD ServerThread:execute
   LOCAL oClient, nClientSocket

   IF ::nSocket <> 0
      nClientSocket := SocketAccept( ::nSocket )

      IF nClientSocket <> 0
         ::logClient( nClientSocket )
         oClient := ClientThread():new( nClientSocket )
         oClient:start()
      ENDIF
   ENDIF
RETURN self


/*
 * Thread is ending!
 */
METHOD ServerThread:atEnd
   ::nSocket := 0
RETURN self


/*
 * Stop listening thread from outside!
 *
 * :setInterval( NIL ) results in :execute() not being called again.
 * SocketClose() causes SocketAccept() -which is blocking- to return.
 * :synchronize(0) causes external function to wait until this thread
 * has ended.
 */
METHOD ServerThread:stop
   IF ::nSocket <> 0
      ::setInterval( NIL )

      SocketClose( ::nSocket )

      ::synchronize(0)
   ENDIF
RETURN self


/*
 * Display log information
 */
METHOD ServerThread:logClient( nClientSocket )
   LOCAL aIPAddress := SocketGetPeerName( nClientSocket )

   ? "Servicing:", Var2Char( aIPAddress )
RETURN self




/* ***************************************************************************
 * This class communicates with a client process using a simple communication
 * protocol. The first 4 bytes of received data contain the length of the
 * subsequent message that must be read. The message consists of binary
 * data that is converted with Bin2Var()/Var2Bin().
 */
CLASS ClientThread FROM Thread
   PROTECTED:
   VAR nSocket
   METHOD evalData

   EXPORTED:
   METHOD init, execute
ENDCLASS


METHOD ClientThread:init( nSocket )
   ::Thread:init()
   ::nSocket := nSocket
RETURN self


/*
 * Establish a simple communication with a client process
 *
 *  1. Read incoming message
 *  2. Bin2Var() -> array of code blocks
 *  3. Evaluate the code blocks
 *  4. Var2Bin() -> result is binary
 *  5. Send binary result to client process
 */

METHOD ClientThread:execute
   LOCAL nError  := 0
   LOCAL cLength := Space(4)
   LOCAL cBuffer := '', cBuff
   LOCAL nBytes := 4
   LOCAL xData
   LOCAL bError
   LOCAL oError  := NIL
   LOCAL cError
   LOCAL cInfo, i

   DO WHILE nBytes <> 0

      nBytes := SocketRecv( ::nSocket, @cLength, 4, @nError )
      IF nError <> 0 .OR. nBytes <> 4
          cBuffer := "ERROR:" + LTrim(Str(nError)) + " BYTES RECEIVED:" + LTrim(Str(nBytes))
          cLength := U2Bin( Len( cBuffer ) )
          SocketSend( ::nSocket, Chr(21) )
          Sleep( 5 )
          SocketSend( ::nSocket, cLength + cBuffer )
          RETURN self
      ENDIF

      /*
        * Read message from socket
        */

      nBytes  := Bin2U( cLength )
      cBuff := Space( nBytes )
      IF nBytes > 0
        nBytes  := SocketRecv( ::nSocket, @cBuff, nBytes, ,@nError )
        cBuffer += cBuff
        SocketSend( ::nSocket, chr(6)  )
      ENDIF

   ENDDO

   IF nError == 0
      /*
       * CLIENT.EXE sends an array of code bocks!
       * Convert array, evaluate the code blocks
       * and return result as binary data.
       */
      bError := ErrorBlock( {|e| break(e) } )
      BEGIN SEQUENCE
         xData   := Bin2Var( cBuffer )
         aLogin := xData[1]
         IF Valtype(aLogin[1]) # 'C' .OR. Upper(aLogin[1]) # 'EXPRESS16'
           cBuffer := Var2Bin('ERROR: Login Invalid')
         ELSEIF Valtype(aLogin[2]) # 'C' .OR. aLogin[2] # 'GUIMonster'
           cBuffer := Var2Bin('ERROR: Login Invalid')
         ELSE
           cBuffer := NIL
           aBlocks := xData[2]
           xData   := ::evalData( aBlocks )
           cBuffer := Var2Bin( xData )
         ENDIF
      RECOVER USING oError
         IF Empty(cBuffer)
         cInfo := oError:description+' '+oError:operation+;
           IIF(!EMPTY(oError:subsystem),;
           "  "+oError:subsystem + "[" + LTrim(Str(oError:subCode)) + "]",'')+;
           IIF(oError:OSCode>0," OS Code["+LTrim(Str(oError:OSCode))+"]",'')
           cBuffer := Var2Bin("ERROR:" + cInfo)
         ENDIF
      END SEQUENCE
      ErrorBlock( bError )
   ELSE
      cBuffer := Var2Bin("ERROR:" + LTrim(Str(nError)))
   ENDIF

   FOR i := 1 to Len( cBuffer ) STEP 1000
     cBuff := Substr( cBuffer, i, 1000  )
     cLength := U2Bin( Len( cBuff ) )
     SocketSend( ::nSocket, cLength + cBuff )
     cBuff := ' '
     SocketRecv( ::nSocket, @cBuff, 1, ,@nError )
     IF cBuff <> Chr(6) .OR. nError <> 0
       ? "ERROR Sending data"
       EXIT
     ENDIF
     // Sleep(5)
   NEXT

   cLength := U2Bin( 0 )
   SocketSend( ::nSocket, cLength  )

// debugging
nHandle := FCreate('SERVER.VAR')
FWrite(nHandle,cBuffer)
FClose(nHandle)

RETURN self


/*
 * aData is a 2-column array -> { { lIsResult, bCodeblock } }
 */
METHOD ClientThread:evalData( aData )
   LOCAL xResult
   LOCAL i, imax := Len( aData )

   FOR i:=1 TO imax
      IF aData[i,1]
         xResult := Eval( aData[i,2] )
      ELSE
         Eval( aData[i,2] )
      ENDIF
   NEXT
RETURN xResult

* ------------------

FUNCTION Scatter( aValues, aFields )

LOCAL i, nField

   IF Valtype( aValues ) # "A"
      aValues := Array( FCount() )
   ENDIF

   IF Valtype( aFields ) # 'A'
      aFields := Array( Fcount() )
      AFields(aFields)
   ENDIF

   aValues := Array(Len(aFields))

   IF !Empty( aValues )
     FOR i := 1 TO Len(aFields)
       nField := FieldPos(aFields[i])
       IF nField > 0
         aValues[i] := FieldGet(nField)
       ELSE
         aValues[i] := &(aFields[i])
       ENDIF
     NEXT
   ENDIF

RETURN aValues

* ------------

FUNCTION Gather( aValues )

IF Valtype( aValues ) == "A"

  IF AScan( aValues, {|x| Valtype(x) <> "O" } ) > 0
    AEval( aValues, {|x,i| FieldPut(i,x) } )
  ELSE
    AEval( aValues, {|o| o:getData() } )
  ENDIF

ENDIF

RETURN aValues

* ---------------

FUNCTION LoadDbes()

LOCAL i, aDbeList := DbeList(), cDbeList := '', cDbe := dbeSetDefault()

IF Valtype(aDbeList) = 'A'
  FOR i := 1 TO Len(aDbeList)
    cDbeList += aDbeList[i,1] + ','
  NEXT
ENDIF

aDbeList := cDbeList

  IF !('DBFDBE'$aDbeList) .AND. !DbeLoad( "DBFDBE",.T.)
     MsgBox( "Database-Engine DBFDBE not loaded" )
  ENDIF
  IF !('NTXDBE'$aDbeList) .AND. !DbeLoad( "NTXDBE",.T.)
     MsgBox( "Database-Engine NTXDBE not loaded"  )
  ENDIF
  IF !('DBFNTX'$aDbeList) .AND. !DbeBuild( "DBFNTX", "DBFDBE", "NTXDBE" )
     MsgBox( "DBFNTX Database-Engine, Could not build engine" )
  ENDIF
  IF !('CDXDBE'$aDbeList) .AND. !DbeLoad( "CDXDBE",.T.)
     MsgBox( "Database-Engine CDXDBE not loaded" )
  ENDIF
  IF !('DBFCDX'$aDbeList) .AND. !DbeBuild( "DBFCDX", "DBFDBE", "CDXDBE" )
     MsgBox( "DBFCDX Database-Engine, Could not build engine" )
  ENDIF
  IF !('FOXDBE'$aDbeList) .AND. !DbeLoad( "FOXDBE",.T.)
     MsgBox( "Database-Engine FOXDBE not loaded" )
  ENDIF
  IF !('FOXCDX'$aDbeList) .AND. !DbeBuild( "FOXCDX", "FOXDBE", "CDXDBE" )
     MsgBox( "FOXCDX Database-Engine, Could not build engine" )
  ENDIF
  IF !('DELDBE'$aDbeList) .AND. !DbeLoad( "DELDBE",.T.)
     MsgBox( "Database-Engine DELDBE not loaded" )
  ENDIF

DbeSetDefault(cDbe)

RETURN .t.


