/*
 ķ
   Program..: CRTFONT.PRG                                
   Author...: Roger J. Donnay                            
   Notice...: (c) DONNAY Software Designs 1987-2000      
   Date.....: Nov 28, 2000                               
   Notes....: Crt Window Font Selection                  
 Ľ

This example program shows how to use a CRTFONT.INI file
to select the font for the CRT Window of a text-based
application.  The user can change the font dynamically by
pressing the ALT-F key.  The changes will be saved to
the CRTFONT.INI file.

*/


#INCLUDE "xbp.ch"
#INCLUDE "inkey.CH"
#INCLUDE "dccolor.CH"
#INCLUDE "dcdialog.CH"

MEMVAR dCCOLOR

FUNCTION Main()

ChrSel()

RETURN nil


PROCEDURE AppSys()

  LOCAL oCrt, nAppType := AppType(), nCol, nRow, oParent, aCrtDef, ;
        aCrtData

  aCrtDef := { 'FontHeight', 'FontWidth', 'FontName' }
  aCrtData := { 12, 8, Pad('Alaska Crt',50) }

  DC_IniLoad( 'CRTFONT.INI',  { { 'CRTWINDOW',  aCrtData,  aCrtDef } } )

      // First active SetAppWindow() == Desktop
  oParent := AppDesktop ( SetAppWindow() )
  nCol := (oParent:currentSize()[1]-620)/2
  nRow := (oParent:currentSize()[2]-400)/2

  // Create XbpCRT object
  oCrt := XbpCrt():New ( oParent, NIL, { nCol, nRow }, 25, 80, nil, .f. )
  oCrt:FontWidth  := aCrtData[2]
  oCrt:FontHeight := aCrtData[1]
  oCrt:FontName   := Alltrim(aCrtData[3])
  oCrt:Title      := "Crt Window ( Press Alt-F to Select Font )"
  oCrt:border     := XBPDLG_RECESSEDBORDERTHICK_FIXED
  oCrt:icon       := DC_IconDefault()
  SetKey( K_ALT_F, {||SelectFont(oCrt)} )

  oCrt:Create()

  // Init Presentation Space
  oCrt:PresSpace()

  DC_CenterObject(oCrt)
  oCrt:show()

  // XbpCrt gets active window and output device
  SetAppWindow( oCrt )
  SetAppFocus( oCrt )

RETURN

* -----------------

STATIC FUNCTION SelectFont( oCrt )

LOCAL aFonts, nPointer := 1, oBrowse, lOk, GetList := {}, aCrtData, aCrtDef, ;
      cFontName, oFont

aFonts := { ;
   { 'Pick a Font', 0, 0 }, ;
   { 'Alaska Crt', 8, 12 }, ;
   { 'Alaska Crt', 8, 14 }, ;
   { 'Alaska Crt', 8, 16 }, ;
   { 'Alaska Crt', 8, 22 }, ;
   { 'Alaska Crt', 8, 28 }, ;
   { 'Lucida Console',  9, 13 }, ;
   { 'Lucida Console', 10, 15 }, ;
   { 'Courier New Bold', 8, 12 }, ;
   { 'Courier New Bold', 8, 14 }, ;
   { 'Courier New Bold', 8, 16 }, ;
   { 'Courier New Bold', 10, 12 }, ;
   { 'Courier New Bold', 10, 14 }, ;
   { 'Courier New Bold', 10, 16 }, ;
   { 'Courier New Bold', 12, 12 }, ;
   { 'Courier New Bold', 12, 14 }, ;
   { 'Courier New Bold', 12, 16 }, ;
   { 'Terminal',   8, 12 }, ;
   { 'Terminal',   8, 16 }, ;
   { 'Terminal',  10, 18 }, ;
   { 'Terminal',  10, 24 } }

@ 0,0 DCBROWSE oBrowse DATA aFonts SIZE 30,11 ;
      DATALINK {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)} ;
      POINTER nPointer FIT PRESENTATION DC_BrowPres()

DCBROWSECOL ELEMENT 1 PARENT oBrowse HEADER 'Font Name' WIDTH 10
DCBROWSECOL ELEMENT 2 PARENT oBrowse HEADER 'Width' WIDTH 4
DCBROWSECOL ELEMENT 3 PARENT oBrowse HEADER 'Height' WIDTH 4

DCREAD GUI FIT ADDBUTTONS TO lOk MODAL TITLE 'Select a CRT Font'

IF lOk

  IF aFonts[nPointer,1] = 'Pick'
    cFontName := DC_PopFont(nil,@oFont)
    IF Valtype(oFont) = 'O'
      aFonts[nPointer,1] := oFont:familyName
      aFonts[nPointer,2] := oFont:width
      aFonts[nPointer,3] := oFont:height
    ELSE
      RETURN nil
    ENDIF
  ENDIF

  oCrt:fontName := aFonts[nPointer,1]
  oCrt:fontWidth := aFonts[nPointer,2]
  oCrt:fontHeight := aFonts[nPointer,3]
  oCrt:configure()
  DC_CenterObject(oCrt)

  aCrtDef := { 'FontHeight', 'FontWidth', 'FontName' }
  aCrtData := { aFonts[nPointer,3], aFonts[nPointer,2], aFonts[nPointer,1] }

  DC_IniSave( 'CRTFONT.INI',  { { 'CRTWINDOW',  aCrtData,  aCrtDef } } )

ENDIF

RETURN nil

* -------------------

#define DC_CHRMSG_1  'DEC CHR '
#define DC_CHRMSG_2  ' PgUp PgDn Home End   ENTER=Select  '

#pragma Ansi2Oem(OFF)

FUNCTION ChrSel( )

LOCAL  nRow, nCol, nCharacter, lPaintScrn, cSaveColor, lExplode,;
       nCharSave, nRowSave, nColSave, cSaveScreen, nRsave, nCsave, ;
       cCode, nInkey, nMouseRow, nMouseCol, aMouseKeys, nPage, ;
       aSetKey, oAppWindow := SetAppWindow(), oCrtWindow

cSaveColor := SETCOLOR()
nRsave := ROW()
nCsave := COL()
cCode := ' '
nInkey := 0
nCharacter := 0
nRow := 5
nCol := 8
nPage := 0
STORE .t. TO lPaintScrn,lExplode
aMouseKeys := { ;
  { 1,10,,,K_ESC },;
  { 22,10,,,K_LEFT },;
  { 22,12,,,K_RIGHT },;
  { 22,14,,,K_UP },;
  { 22,16,,,K_DOWN },;
  { 22,18,22,21,K_PGUP },;
  { 22,23,22,26,K_PGDN },;
  { 22,28,22,31,K_HOME },;
  { 22,33,22,35,K_END },;
  { 22,39,22,49,K_ENTER } }
DO WHILE .T.
  nCharacter := (nRow-4)+16*((nCol/8)-1)-1+nPage
  IF lPaintScrn
    nCharSave := nCharacter
    DC_DispBegin()
    IF lExplode
      cSaveScreen := DC_EXPL(1,7,23,71,,,,,,,.f.)
      lExplode := .f.
      @ 21,8 TO 21,70
      DC_COLOR(nC_EXPBOXF)
      @ 1,9 SAY '[]'
      @ 22,8 CLEAR TO 22,70
      @ 22,10 SAY CHR(27)+' '+CHR(26)+' '+CHR(24)+' '+CHR(25)+DC_CHRMSG_2
      DC_COLOR(nC_EXPBOXC)
      @ 2,8 CLEAR TO 20,70
      * 'DEC CHR '
      @ 2,8 SAY TRIM(REPL(DC_CHRMSG_1,8))
      @ 3,8 SAY TRIM(REPL('--- ',16))
    ENDIF
    DC_COLOR(nC_DBUGTXT)
    IF nPage = 0
      nCharacter := 0
    ELSE
      nCharacter := 128
    ENDIF
    FOR nCol  :=  8 TO 64 STEP 8
      FOR nRow  :=  5 TO 20
        @ nRow,nCol SAY STR(nCharacter,3)+'  '+CHR(nCharacter)
        IF nCharacter=nCharSave
          nRowSave := nRow
          nColSave := nCol
        ENDIF
        nCharacter++
      NEXT
      IF nCol<64
        @ 2,nCol+7 TO 20,nCol+7
      ENDIF
    NEXT
    nRow := nRowSave
    nCol := nColSave
    lPaintScrn := .f.
    nCharacter := nCharSave
    DC_DispEnd()
  ENDIF
  DC_COLOR(nC_DBUGBAR)
  nCharacter := (nRow-4)+16*((nCol/8)-1)-1+nPage
  @ nRow,nCol+3 SAY CHR(17)+CHR(16)
  DC_COLOR(nC_DBUGTXT)
  nInkey := DC_INKEY('',0,@nMouseRow,@nMouseCol)
  cCode := IIF(nInkey<32 .OR. nInkey>300,CHR(255),UPPER(CHR(nInkey)))
  @ nRow,nCol+3 SAY '  '

  DO CASE

    CASE nInkey <= -101 .AND. nMouseRow>4 .AND. nMouseRow<21 ;
                       .AND. nMouseCol>7 .AND. nMouseCol<73
      nRow := nMouseRow
      nCol := INT( nMouseCol/8 ) * 8
      IF nInkey = -104
        KEYBOARD CHR(13)
      ENDIF

    CASE (cCode='2' .OR. nInkey=24)  &&Down arrow key
      IF nRow=20 .AND. nCol=64
        LOOP
      ENDIF
      nRow++
      IF nRow>20
        nRow := 5
        nCol := nCol+8
      ENDIF

    CASE (cCode='8' .OR. nInkey=K_UP)
      IF nRow=5 .AND. nCol=8
        LOOP
      ENDIF
      nRow--
      IF nRow<5
        nRow := 20
        nCol := nCol-8
      ENDIF

    CASE (cCode='6' .OR. nInkey=K_RIGHT) .AND. nCol<64
      nCol := nCol+8

    CASE (cCode='4' .OR. nInkey=K_LEFT) .AND. nCol>8
      nCol := nCol-8

    CASE (nInkey=K_PGDN .OR. cCode='3') .OR. (nInkey=K_PGUP .OR. cCode='9')
      lPaintScrn := .t.
      IF nPage = 0
        nPage := 128
      ELSE
        nPage := 0
      ENDIF

    CASE (nInkey=K_HOME .OR. cCode='7')
      nRow := 5
      nCol := 8

    CASE (nInkey=K_END .OR. cCode='1')
      nRow := 20
      nCol := 64

    CASE nInkey=K_ESC .OR. nInkey=13
      DC_IMPL(cSaveScreen)
      DC_COLOR(cSaveColor)
      @ nRsave,nCsave SAY ''
      EXIT
  ENDCASE
ENDDO

RETURN nil


