
#include "Gra.ch"
#include "Xbp.ch"
#include "Appevent.ch"
#include "Font.ch"
#include "thread.ch"
#include "dcbitmap.CH"
#include "dcdialog.CH"
#include "dctree.ch"
#include "dcicon.ch"

MEMVAR help_tree

#define HELP_LOOKUP xbeP_User + 1

#define HELP_TYPE_TEXTMEMO      1
#define HELP_TYPE_TEXTFILE      2
#define HELP_TYPE_TEXTCONTROL   3

// #define TXCONTROL

#ifdef TXCONTROL
  #pragma Library("JACP30XPPCOFF.LIB")
  #include "TXCONTR.CH"
  #include "TXCONTR.PRG"
  #include "JACE30XPP.ch"
#endif

#define LEFT_MARKER  '**'
#define RIGHT_MARKER  '**'

CLASS Help

EXPORTED:
  VAR helpCode
  VAR userID
  VAR userName
  VAR description
  VAR TxControl
  VAR TxButtonBar
  VAR TxRuler
  VAR TxStatusBar
  VAR getList
  VAR getOptions
  VAR TxDlgArea
  VAR seekCode
  VAR mainDialog
  VAR status
  VAR editMenu
  VAR textHeight
  VAR editMode
  VAR parent
  VAR refresh
  VAR type
  VAR memo
  VAR memoObject
  VAR font
  VAR title
  VAR staticObject
  VAR focusHelp
  VAR motionHelp
  VAR treeRoot
  VAR helpRoot
  VAR lastMarked
  VAR showTree
  VAR motionDelay

  METHOD init, dialog, openFiles, LookUp, Find, RebuildFindIndex, ;
         CreateTxControl, Load, Save, Destroy, Delete, buildTreeIndex, ;
         buildTree, ResizeWindow, editTree, saveTreeItem, ;
         deleteTreeItem

ENDCLASS

* ----------------

METHOD Help:init( lEdit, oParent, nType, cFont, cTitle, cHelpRoot )

LOCAL oThread := Thread():new()

::status = 0
::editMode := lEdit
::parent := oParent
::refresh := .f.
::type := nType
::memo := ''
::font := cFont
::title := cTitle
::focusHelp := .f.
::motionHelp := .f.
::helpRoot := cHelpRoot
::showTree := .t.
::motionDelay := 30

DEFAULT ::editMode := .t., ;
        ::type := HELP_TYPE_TEXTMEMO, ;
        ::font := '10.Arial', ;
        ::title := 'Context Help System', ;
        ::helpRoot := 'DESKTOP'

oThread:start( {||::Dialog()} )
Sleep(20)

RETURN self

* ----------------

METHOD Help:Dialog()

LOCAL oToolBar, GetList[0], oDlg, GetOptions, lStartUp := .t.

_LoadRdds()
::OpenFiles()
::getList := GetList
::status := 1

IF ::editMode .AND. ::type == HELP_TYPE_TEXTCONTROL
  DCSUBMENU ::editMenu PROMPT '&Edit'
  DCMENUITEM '&Cut' ACTION {||::TxControl:clip(1)} PARENT ::editMenu
  DCMENUITEM 'Cop&y' ACTION {||::TxControl:clip(2)} PARENT ::editMenu
  DCMENUITEM '&Paste' ACTION {||::TxControl:clip(3)} PARENT ::editMenu
ENDIF

::helpCode := Space(50)
::description := Space(50)

@ 0,30 DCSTATIC TYPE XBPSTATIC_TYPE_RECESSEDBOX SIZE 80,2.5 ;
      OBJECT ::staticObject

@ .2,1 DCSAY '  Help Code' GET ::helpCode PICT '@!' SAYRIGHT ;
      COMBO HEIGHT 10 DATA 'HELP' ;
      FIELD {||Pad(HELP->help_code,15) + ': ' + HELP->desc} ;
      RETURN {||::refresh:=.t.,HELP->help_code} ;
      GOTFOCUS {||IIF(::refresh,::Load(),nil), ;
                  ::refresh:=.f.} ;
      PARENT ::staticObject

@ 1.2,1 DCSAY 'Description' GET ::description SAYRIGHT ;
      PARENT ::staticObject

@ 2.5,30 DCDIALOG oDlg NOTITLEBAR DRAWINGAREA ::TxDlgArea SIZE 80,21

@ 0,0 DCTREEROOT OBJECT ::treeRoot SIZE 30,23.5 HASLINES HASBUTTONS ;
      ITEMMARKED {|o|IIF(Valtype(::lastMarked)=='O',;
                   (::lastMarked:setCaption(Strtran(::lastMarked:caption,LEFT_MARKER,'')), ;
                    ::lastMarked:setCaption(Strtran(::lastMarked:caption,RIGHT_MARKER,''))),nil),;
                    ::lookUp(o:cargo[3,1],,.f.), ;
                    o:setCaption(LEFT_MARKER+Trim(o:caption)+RIGHT_MARKER),;
                    ::lastMarked := o}

::buildTree( @::treeRoot, ::helpRoot, .t. )

@ 24,0 DCTOOLBAR oToolBar SIZE 62,1.2 BUTTONSIZE 9,1.2 ;
       TYPE XBPSTATIC_TYPE_TEXT

@ 23.5,67 DCCHECKBOX ::focusHelp PROMPT 'Focus Help'
@ 24.2,67 DCCHECKBOX ::motionHelp PROMPT 'Motion Help'
@ 24.9,67 DCCHECKBOX ::showTree PROMPT 'Show Tree' ;
          EVAL {|o|o:selected := {||::resizeWindow()}}

DCADDBUTTON CAPTION {BITMAP_TOP_M, BITMAP_TOP_DISABLED_M} ;
   ACTION {||DC_DbGoTop(),::Load()} PARENT oToolBar ;
   WHEN {||!HELP->(DC_TestBof())} ;
   SIZE 4,1.2

DCADDBUTTON CAPTION {BITMAP_PREVIOUS_M, BITMAP_PREVIOUS_DISABLED_M} ;
   ACTION {||DC_DbSkip(-1),::Load()} PARENT oToolBar ;
   WHEN {||!HELP->(DC_TestBof())} ;
   SIZE 4,1.2

DCADDBUTTON CAPTION {BITMAP_NEXT_M, BITMAP_NEXT_DISABLED_M} ;
   ACTION {||DC_DbSkip(1),::Load()} PARENT oToolBar ;
   WHEN {||!HELP->(DC_TestEof())} ;
   SIZE 4,1.2

DCADDBUTTON CAPTION {BITMAP_BOTTOM_M, BITMAP_BOTTOM_DISABLED_M} ;
   ACTION {||DC_DbGoBottom(),::Load()} PARENT oToolBar ;
   WHEN {||!HELP->(DC_TestEof())} ;
   SIZE 4,1.2

IF !(::type == HELP_TYPE_TEXTCONTROL)
   DCADDBUTTON CAPTION 'Font' ;
      PARENT oToolBar ;
      ACTION {||::font := DC_PopFont(::font), ;
                ::memoObject:setFontCompoundName(::font)}
ENDIF

DCADDBUTTON CAPTION 'Find' ACTION {|c|c:=::Find(),IIF(!Empty(c),::LookUp(c),nil)} ;
   PARENT oToolBar

IF ::editMode
  DCADDBUTTON CAPTION 'Save' ACTION {||::Save()} PARENT oToolBar
  DCADDBUTTON CAPTION 'Delete' ACTION {||::Delete()} PARENT oToolBar
ENDIF

DCADDBUTTON CAPTION 'Exit' ;
   ACTION {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)} PARENT oToolBar

DCHOTKEY xbeK_ALT_T ACTION {||::editTree()}

DCGETOPTIONS ;
   TABSTOP ;
   ABORTQUERY MSG {||::mainDialog:hide(),.f.} ;
   EXITQUERY MSG {||::mainDialog:hide(),.f.} ;
   CLOSEQUERY MSG {||::mainDialog:hide(),.f.}

::getOptions := GetOptions

DCREAD GUI FIT;
   HANDLER _HelpHandler REFERENCE self ;
   OPTIONS GetOptions ;
   TITLE ::title ;
   EVAL {|o|::mainDialog := o, ;
            ::CreateTxControl(), ;
            o:resize := {|a,b|::ResizeWindow(a,b)}, lStartup := .f.}

CLOSE ALL

RETURN self

* -----------------

METHOD Help:ResizeWindow( aOldSize, aNewSize )

LOCAL oTxDlgArea := ::TxDlgArea, oTree := ::treeRoot, ;
      lResize := !Empty(aOldSize)

LOCAL oDlg := oTxDlgArea:setParent()

DEFAULT aOldSize := oDlg:currentSize(), ;
        aNewSize := aOldSize

oDlg:setSize( { oDlg:currentSize()[1]+(aNewSize[1]-aOldSize[1]), ;
                oDlg:currentSize()[2]+(aNewSize[2]-aOldSize[2]) } )

::memoObject:setSize( { oDlg:currentSize()[1]-6, oDlg:currentSize()[2]-6 } )
::memoObject:setPos({0,0})

::staticObject:setPos( { ::staticObject:currentPos()[1], ;
                         ::staticObject:currentPos()[2]+(aNewSize[2]-aOldSize[2]) } )

IF !lResize

  IF ::showTree
    oTree:show()
    ::staticObject:setPos( { oTree:currentPos()[1] + oTree:currentSize()[1], ;
                             ::staticObject:currentPos()[2] } )
    oDlg:setPos( { oTree:currentPos()[1] + oTree:currentSize()[1], ;
                   oDlg:currentPos()[2] } )
  ELSE
    oTree:hide()
    ::staticObject:setPos( { 10, ::staticObject:currentPos()[2] } )
    oDlg:setPos( { 10, oDlg:currentPos()[2] } )
  ENDIF

  ::mainDialog:setSize( { oDlg:currentPos()[1]+oDlg:currentSize()[1]+20, ;
                          ::mainDialog:currentSize()[2] } )

  DC_ClearEvents()

ENDIF


RETURN nil

* ------------------

METHOD Help:editTree()

LOCAL Getlist[0], cHelpTree := Space(50), cHelpTitle := Space(50), ;
      oBrowse, oToolBar, GetOptions

::openFiles()

@ 0,0 DCBROWSE oBrowse ALIAS 'HELPTREE' SIZE 80,20 ;
      PRESENTATION DC_BrowPres() ;
      NOHSCROLL ;
      ITEMMARKED {|| cHelpTree := HELPTREE->help_tree, ;
                     cHelpTitle := HELPTREE->help_title, ;
                     DC_GetRefresh( GetList, nil, ;
                       DCGETREFRESH_TYPE_EXCLUDE, {GETLIST_BROWSE} ) }

DCBROWSECOL FIELD HELPTREE->help_tree HEADER 'Tree.Code' WIDTH 20 PARENT oBrowse
DCBROWSECOL FIELD HELPTREE->help_title HEADER 'Title' WIDTH 30 PARENT oBrowse

@ 21,0 DCSAY 'Help Tree (Parent.Code)' GET cHelpTree PICT '@!' ;
      VALID {||'.'$cHelpTree}

@ 22,0 DCSAY 'Help Title ' GET cHelpTitle

@ 24,0 DCTOOLBAR oToolBar SIZE 40,1.2 BUTTONSIZE 10,1.2

DCADDBUTTON CAPTION 'New' PARENT oToolBar ;
   ACTION {||cHelpTree := Space(50), cHelpTitle := Space(50), DC_GetRefresh(GetList)}

DCADDBUTTON CAPTION 'Save' PARENT oToolBar ;
   ACTION {||::saveTreeItem( cHelpTree, cHelpTitle ), DC_GetRefresh(GetList)}

DCADDBUTTON CAPTION 'Delete' PARENT oToolBar ;
   ACTION {||::deleteTreeItem( cHelpTree, cHelpTitle ), DC_GetRefresh(GetList)}

DCADDBUTTON CAPTION 'Exit' PARENT oToolBar ;
   ACTION {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)}

DCGETOPTIONS SAYWIDTH 150 SAYRIGHT

DCREAD GUI FIT MODAL TITLE 'Help Tree Editor' OPTIONS GetOptions

RETURN self

* ------------------

STATIC FUNCTION _HelpHandler( nEvent, mp1, mp2, oXbp, oDlg, aGetList, oHelp )

IF nEvent == HELP_LOOKUP
  oHelp:mainDialog:show()
  oHelp:seekCode := mp1
  oHelp:load(mp1,mp2)
ENDIF

RETURN DCGUI_NONE

* ------------------

METHOD Help:saveTreeItem( cHelpTree, cHelpTitle )

::openFiles()

SELE HELPTREE
HELPTREE->(OrdSetFocus('HELPTREE'))

HELPTREE->(dbSeek(cHelpTree))
IF HELPTREE->(Eof())
  SEEK ' '
ENDIF

IF HELPTREE->(Eof()) .AND. HELPTREE->(AddRec())
   REPL HELPTREE->help_tree WITH cHelpTree, ;
        HELPTREE->help_title WITH cHelpTitle
   HELPTREE->(dbUnlock())
ELSEIF HELPTREE->(RecLock())
   REPL HELPTREE->help_title WITH cHelpTitle, ;
        HELPTREE->help_tree WITH cHelpTree
   HELPTREE->(dbUnlock())
ENDIF

RETURN self

* ------------------

METHOD Help:deleteTreeItem()

IF !HELPTREE->(Eof()) .AND. HELPTREE->(RecLock())
   REPL HELPTREE->help_tree WITH '', ;
        HELPTREE->help_title WITH ''
   HELPTREE->(dbUnlock())
ENDIF

RETURN self

* ------------------

METHOD Help:buildTreeIndex( aGetList )

LOCAL cHelpTree, i

::openFiles()

SELE HELPTREE
HELPTREE->(OrdSetFocus('HELPTREE'))

FOR i := 1 TO Len(aGetList)
  cHelpTree := aGetList[i,cGETLIST_HELPCODE]
  IF !Empty(cHelpTree)
    HELPTREE->(dbSeek(cHelpTree))
    IF HELPTREE->(Eof()) .AND. HELPTREE->(AddRec())
      REPL HELPTREE->help_tree WITH cHelpTree
      HELPTREE->(dbUnlock())
    ENDIF
  ENDIF
NEXT

RETURN self

* ------------------

METHOD Help:buildTree( oTree, cHelpTree, lExpand )

LOCAL GetList[0], oTreeItem, bAction, xCargo, nSaveRec

IF Valtype(oTree) # 'O'
  GetList := ::getList
ENDIF

HELPTREE->(OrdSetFocus('HELPTREE'))
HELPTREE->(dbSeek(cHelpTree))

DO WHILE HELPTREE->help_tree = cHelpTree + '.'

  bAction := { |o|::buildTree( o, o:cargo[3,1] ) }
  xCargo := { Alltrim(DC_TokenArray(HELPTREE->help_tree,'.')[2]), HELPTREE->help_tree}
  DCTREEITEM CAPTION HELPTREE->help_tree ;
       PARENT oTree ;
       CARGO xCargo ;
       OBJECT oTreeItem ;
       ACTION bAction FIRSTCLICK ;
       IMAGENORMAL ICON_CLOSEDFOLDER ;
       IMAGEEXPANDED ICON_OPENFOLDER ;
       IMAGEMARKED ICON_OPENFOLDER

  IF lExpand
    nSaveRec := HELPTREE->(RecNo())
    ::buildTree( @oTreeItem, xCargo[1], lExpand )
    HELPTREE->(dbGoTo(nSaveRec))
  ENDIF

  HELPTREE->(DC_dbSkip())

ENDDO

IF Valtype(oTree) = 'O'
  DCREAD GUI EXIT PARENT oTree
  oTree:expand(.t.)
ENDIF

RETURN self

* ------------------

METHOD Help:find()

LOCAL GetList[0], cWord := Space(20), aFind[0], oBrowse, nLen, nPointer := 0, ;
      cHelpCode

@ 1,1 DCSAY 'Word to Find' GET cWord PICT '@!' SAYRIGHT
@ 1,40 DCPUSHBUTTON CAPTION 'Rebuild Index' SIZE 12 ;
       ACTION {||::RebuildFindIndex()}

DCREAD GUI FIT MODAL ADDBUTTONS ENTEREXIT TITLE 'Find Help'

IF !Empty(cWord)
  HELP->(OrdSetFocus('HELP_CODE'))
  SELE HELPFIND
  HELPFIND->(dbSeek(cWord))
  nLen := Len(Trim(cWord))
  DO WHILE Left(HELPFIND->keyword,nLen) == Trim(cWord)
    HELP->(dbSeek(HELPFIND->help_code))
    AAdd(aFind,{HELPFIND->keyword,HELPFIND->help_code,HELP->desc})
    HELPFIND->(DC_DbSkip())
  ENDDO
  @ 0,0 DCBROWSE oBrowse DATA aFind SIZE 70,20 FIT ;
        POINTER nPointer PRESENTATION DC_BrowPres() ;
        ITEMSELECTED {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)}
  DCBROWSECOL ELEMENT 1 PARENT oBrowse HEADER 'Word' WIDTH 10
  DCBROWSECOL ELEMENT 2 PARENT oBrowse HEADER 'Help Code' WIDTH 10
  DCBROWSECOL ELEMENT 3 PARENT oBrowse HEADER 'Description' WIDTH 40
  DCREAD GUI FIT MODAL ADDBUTTONS TITLE 'Select Help Screen to View'
ENDIF

cHelpCode := ''
IF nPointer > 0
  cHelpCode := aFind[nPointer,2]
ENDIF

RETURN cHelpCode

* ------------------

METHOD Help:RebuildFindIndex()

LOCAL i, cHelpCode, cText, aTokens, cWord, oScrn := DC_WaitOn('Building Find Index')

SELE HELPFIND
HELPFIND->(OrdSetFocus('KEYWORD'))
HELP->(OrdSetFocus(0))
HELP->(DC_DbGoTop())
DO WHILE HELP->(!Eof())
  BEGIN SEQUENCE
  cHelpCode := HELP->help_code
  IF ::type == HELP_TYPE_TEXTCONTROL
    ::TxControl:resetContents()
    ::TxControl:Load(Alltrim(cHelpCode)+'.RTF',0,5)
    cText := HELP->desc + ' ' + ::TxControl:text
  ELSEIF ::type == HELP_TYPE_TEXTMEMO
    cText := HELP->desc + ' ' + HELP->memo
  ELSEIF ::type == HELP_TYPE_TEXTFILE
    cText := HELP->desc + ' ' + MemoRead(Alltrim(cHelpCode)+'.TXT')
  ENDIF
  aTokens := DC_TokenArray(cText,' .;:,-"[]'+Chr(13)+Chr(10))

  FOR i := 1 TO Len(aTokens)
    cWord := Upper(Alltrim(aTokens[i]))
    IF Empty(cWord)
      LOOP
    ELSEIF cWord $ {'AT','THE','OF','A','BE','TO','IS','ALL','THAT','THIS', ;
                    'WILL','AS','FOR','EACH','IT','NOW','DO','IN','AND','OR', ;
                    'ANY','YOU','HAVE','HAS','ON','OUR','BY','NOT','AN','FROM', ;
                    'YOUR','WITH','HAD'}
      LOOP
    ELSEIF Val(cWord) > 0
      LOOP
    ENDIF
    cWord := Pad(cWord,20)
    HELPFIND->(DbSeek(cWord+cHelpCode))
    IF HELPFIND->(Eof()) .AND. HELPFIND->(AddRec())
      REPL HELPFIND->keyword WITH cWord, ;
           HELPFIND->help_code WITH cHelpCode
      HELPFIND->(dbUnlock())
    ENDIF
  NEXT
  END SEQUENCE
  HELP->(DC_DbSkip())
ENDDO

DC_Impl(oScrn)
DC_MsgBox('Done!')

RETURN self

* ------------------

METHOD Help:destroy()

IF ::type == HELP_TYPE_TEXTCONTROL
  ::TxControl:Disconnect()
  IF ::editMode
    ::TxButtonBar:Disconnect()
    ::TxRuler:Disconnect()
    ::TxStatusBar:Disconnect()
  ENDIF
ENDIF

::TxControl := nil
::TxButtonbar := nil
::TxRuler := nil
::TxStatusBar := nil
::status := 0
::mainDialog := nil
::TxDlgArea := nil
::getList := nil
::getOptions := nil
::editMenu := nil
::helpcode := nil
::description := nil

RETURN self

* ------------------

METHOD Help:LookUp( cCode, cTitle, lTree )

LOCAL cHelpCode, oTreeItem

DEFAULT lTree := .t.

IF lTree .AND. ::showTree
  oTreeItem := _GetTreeItem(::treeRoot:rootItem,cCode)
  IF Valtype(oTreeItem) == 'O'
    ::treeRoot:setData(oTreeItem)
  ENDIF
ENDIF

IF '.' $ cCode
  cHelpCode := DC_TokenArray(cCode,'.')[2]
ELSE
  cHelpCode := cCode
ENDIF

PostAppEvent( HELP_LOOKUP, cHelpCode, cTitle, ::mainDialog )

RETURN self

* ------------------

STATIC FUNCTION _GetTreeItem(oTree,cHelpCode)

LOCAL i, aChildList := oTree:getChildItems(), oTreeItem, aSubChildList

FOR i := 1 TO Len(aChildList)
  IF Alltrim(aChildList[i]:cargo[3,2]) == cHelpCode
    RETURN aChildList[i]
  ENDIF
  aSubChildList := aChildList[i]:getChildItems()
  IF !Empty(aSubChildList)
    oTreeItem := _GetTreeItem(aChildList[i],cHelpCode,aSubChildList)
    IF Valtype(oTreeItem) == 'O'
      RETURN oTreeItem
    ENDIF
  ENDIF
NEXT

RETURN nil

* ------------------

METHOD Help:CreateTxControl()

LOCAL oThread,w,h,hruler,hbutton,htext,hstatus, xWhatEver, cKeyCode, ;
      GetList[0]

IF ::type == HELP_TYPE_TEXTCONTROL

  #ifdef TXCONTROL
  oThread := Thread():new()
  oThread:start( "JAProcessOCXEvents")

  w:=::TxDlgArea:currentSize()[1]
  h:=::TxDlgArea:currentSize()[2]
  hruler := 32
  hButton := 32
  hstatus := 20
  ::textHeight := h-(hruler+hbutton+hstatus)

  cKeyCode := "ProgID:TX.TextControl.110;" + ;
              "JARTKEY:Ewwz6A-48ACE-57321-FC151-5"

  ::TxControl:= TXTextControl():new()
  xWhatEver:=JAXPPCREATECONTROL(::TxDlgArea:getHWND(),"TXControl1",0,hstatus,w,::textHeight,cKeyCode,1)
  ::TxControl:Connect(xWhatEver)
  ::TxControl:ScrollBars := 3
  ::TxControl:FontName := "Courier New"
  ::TxControl:FontSize := 10
  ::TxControl:pageWidth := 12000
  ::TxControl:pageHeight := 15000

  IF ::editMode

    // Buttonbar
    ::TxButtonBar := TXButtonBar():new()
    xWhatEver := JAXPPCREATECONTROL(::TxDlgArea:getHWND(),"TXButtonBarControll",0,::textHeight+hstatus+hruler,w,hbutton,"ProgID:TX.ButtonBarControl.110",1)
    ::TxButtonBar:Connect(xWhatEver)
    JAXPPSETCONTROLRESIZESTYLE(xWhatEver,JAHANDLE_PUNK,JARESIZESTYLE_X2RIGHT)

    // Ruler
    ::TxRuler := TXRuler():new()
    xWhatEver := JAXPPCREATECONTROL(::TxDlgArea:getHWND(),"TXRuler1",0,::textHeight,hstatus,w,hruler,"ProgID:TX.RulerControl.110",1)
    ::TxRuler:Connect(xWhatEver)
    JAXPPSETCONTROLRESIZESTYLE(xWhatEver,JAHANDLE_PUNK,JARESIZESTYLE_X2RIGHT)

    // Statusbar
    ::TxStatusBar := TXStatusBar():new()
    xWhatEver := JAXPPCREATECONTROL(::TxDlgArea:getHWND(),"TXStatusBar1",0,0,w,hstatus,"ProgID:TX.StatusBarControl.110",1)
    ::TxStatusBar:Connect(xWhatEver)
    JAXPPSETCONTROLRESIZESTYLE(xWhatEver,JAHANDLE_PUNK,JARESIZESTYLE_X2RIGHT)

    ::TxControl:ButtonBarHandle := ::TxButtonBar:hWnd
    ::TxControl:RulerHandle     := ::TxRuler:hWnd
    ::TxControl:StatusBarHandle := ::TxStatusBar:hWnd

    ::TxButtonBar:Enabled := .T.
    ::TxRuler:Enabled := .T.
    ::TxStatusBar:Enabled := .T.

    // Connect to the events fired by the ActiveX.
    ::TxControl:SubscribeToEvents({|hEvent| ProcessEvent(hEvent,self) })

  ENDIF
  #endif

ELSE

   @ 0,0 DCMULTILINE ::memo ;
         TABSTOP ;
         NOHORIZSCROLL ;
         FONT ::font ;
         PIXEL ;
         OBJECT ::memoObject ;
         SIZE ::TxDlgArea:currentSize()[2], ;
              ::TxDlgArea:currentSize()[1]

   DCREAD GUI PARENT ::TxDlgArea EXIT SAVE
   DC_MergeGetLists(::getList,GetList)

ENDIF

::Load()

RETURN (NIL)

* ----------------

// REQUIRED by the JA runtime to process smoothly the OCX events sent by the various ActiveX instances.
#ifdef TXCONTROL
PROCEDURE JAProcessOCXEvents()
  JAXPPPROCESSEVENTS()
RETURN
#endif

* --------------

METHOD Help:Save()

LOCAL aBytes, nSaveOrder := OrdSetFocus(), lDeleted := Set(_SET_DELETED,.f.)

BEGIN SEQUENCE

OrdSetFocus('HELP_CODE')

HELP->(dbSeek(Alltrim(::helpCode)))

IF HELP->(Eof())
  HELP->(dbSeek('    ')) // resuse any empty records
ENDIF
IF HELP->(Eof())
  IF !AddRec()
    BREAK
  ENDIF
ELSE
  IF !RecLock()
    BREAK
  ENDIF
ENDIF

REPL HELP->help_code WITH ::helpCode, ;
     HELP->desc WITH ::description, ;
     HELP->date WITH Date(), ;
     HELP->time WITH Time()

HELP->(dbRecall())

IF ::type == HELP_TYPE_TEXTCONTROL
  FErase(Alltrim(::helpCode)+'.RTF')
  ::TxControl:save(Alltrim(::helpCode)+'.RTF',0,5)
ELSEIF ::type == HELP_TYPE_TEXTMEMO
  REPL HELP->memo WITH ::memo
ELSEIF ::type == HELP_TYPE_TEXTFILE
  MemoWrit(Alltrim(::helpCode)+'.TXT',::memo)
ENDIF

UNLOCK

END SEQUENCE

Set(_SET_DELETED,lDeleted)

RETURN nil

* --------------

METHOD Help:Delete()

LOCAL aBytes, nSaveOrder := OrdSetFocus(), lStatus := .f.

BEGIN SEQUENCE

IF !DC_MsgBox(,,{'Are you sure?'},,,,.t.)
  BREAK
ENDIF

OrdSetFocus('HELP_CODE')

HELP->(dbSeek(Alltrim(::helpCode)))

IF !Eof()
  IF !RecLock()
    BREAK
  ENDIF
ENDIF

REPL HELP->help_code WITH '', ;
     HELP->desc WITH ''

IF ::type == HELP_TYPE_TEXTCONTROL
  FErase(Alltrim(::helpCode)+'.RTF')
ELSEIF ::type == HELP_TYPE_TEXTMEMO
  REPL HELP->memo WITH ''
ELSEIF ::type == HELP_TYPE_TEXTFILE
  FErase(Alltrim(::helpCode)+'.TXT')
ENDIF

HELP->(dbDelete())
lStatus := .t.

UNLOCK

END SEQUENCE

RETURN lStatus

* ----------------

METHOD Help:Load( cHelpCode, cTitle )

IF !Empty(::seekCode)
  HELP->(dbSeek(::seekCode))
  ::helpCode := ::seekCode
ELSEIF !Empty(cHelpCode)
  ::helpCode := cHelpCode
ELSE
  ::helpCode := HELP->help_code
ENDIF
IF !Empty(cTitle) .AND. Empty(HELP->desc)
  ::description := cTitle
ELSE
  ::description := HELP->desc
ENDIF

IF ::type == HELP_TYPE_TEXTCONTROL
  ::TxControl:resetContents()
  ::TxControl:Load(Alltrim(::helpCode)+'.RTF',0,5)
ELSEIF ::type == HELP_TYPE_TEXTMEMO
  ::memo := HELP->memo
ELSEIF ::type == HELP_TYPE_TEXTFILE
  ::memo := MemoRead(Alltrim(::helpCode)+'.TXT')
ENDIF

DC_GetRefresh(::getList)
::seekCode := ''

RETURN nil

* -------------

METHOD Help:openFiles()

LOCAL aStructure

_LoadRdds()

IF !DC_dbSel('HELP')
  IF !File('HELP.DBF')
    aStructure := { ;
     { 'HELP_CODE','C',   50, 0 },;
     { 'DESC',     'C',   50, 0 },;
     { 'DATE',     'D',    8, 0 },;
     { 'TIME',     'C',    8, 0 },;
     { 'MEMO',     'M',    4, 0 },;
     { 'AUTHOR',   'C',   20, 0 }}
    dbCreate( 'HELP.DBF', aStructure, 'FOXCDX' )
  ENDIF

  IF !File('HELP.CDX')
    USE HELP VIA 'FOXCDX' NEW EXCLUSIVE
    INDEX ON HELP->help_code TAG 'HELP_CODE' TO HELP.CDX
    SET INDEX TO HELP.CDX
  ELSE
    USE HELP VIA 'FOXCDX' NEW SHARED INDEX HELP.CDX
  ENDIF
ENDIF

IF !DC_dbSel('HELPFIND')
  IF !File('HELPFIND.DBF')
    aStructure := { ;
     { 'HELP_CODE','C',   50, 0 },;
     { 'KEYWORD',  'C',   20, 0 }}
    dbCreate( 'HELPFIND.DBF', aStructure, 'FOXCDX' )
  ENDIF

  IF !File('HELPFIND.CDX')
    USE HELPFIND VIA 'FOXCDX' NEW EXCLUSIVE
    INDEX ON HELPFIND->keyword+HELPFIND->help_code TAG 'KEYWORD' TO HELPFIND.CDX
    SET INDEX TO HELPFIND.CDX
  ELSE
    USE HELPFIND VIA 'FOXCDX' NEW SHARED INDEX HELPFIND.CDX
  ENDIF
ENDIF

IF !DC_dbSel('HELPTREE')
  IF !File('HELPTREE.DBF')
    aStructure := { ;
     { 'HELP_TREE','C',   50, 0 }, ;
     { 'HELP_TITLE','C',  50, 0 }  }
    dbCreate( 'HELPTREE.DBF', aStructure, 'FOXCDX' )
  ENDIF

  IF !File('HELPTREE.CDX')
    USE HELPTREE VIA 'FOXCDX' NEW EXCLUSIVE
    INDEX ON help_tree TAG 'HELPTREE' TO HELPTREE.CDX
    SET INDEX TO HELPTREE.CDX
  ELSE
    USE HELPTREE VIA 'FOXCDX' NEW SHARED INDEX HELPTREE.CDX
  ENDIF
ENDIF

SELE HELP

RETURN nil

* --------------

STATIC FUNCTION _LoadRdds()

LOCAL i, aDbeList := DbeList(), cDbeList := '', cDbe := dbeSetDefault()

IF Valtype(aDbeList) = 'A'
  FOR i := 1 TO Len(aDbeList)
    cDbeList += aDbeList[i,1] + ','
  NEXT
ENDIF

aDbeList := cDbeList

  IF !('CDXDBE'$aDbeList) .AND. !DbeLoad( "CDXDBE",.T.)
     DC_WinAlert( "Database-Engine CDXDBE not loaded" )
  ENDIF
  IF !('FOXDBE'$aDbeList) .AND. !DbeLoad( "FOXDBE",.T.)
     DC_WinAlert( "Database-Engine FOXDBE not loaded" )
  ENDIF
  IF !('FOXCDX'$aDbeList) .AND. !DbeBuild( "FOXCDX", "FOXDBE", "CDXDBE" )
     DC_WinAlert( "FOXCDX Database-Engine, Could not build engine" )
  ENDIF

DbeSetDefault(cDbe)

RETURN .t.

* ----------------

#ifdef TXCONTROL
PROCEDURE ProcessEvent( nEvent, oHelp )

LOCAL xValue, nCol, nRow
LOCAL cEventName := JAXPPGETEVENTNAME(nEvent)

IF cEventName == 'MouseDown' .AND. oHelp:editMode
  xValue := JAXPPGETEVENTPARAMETERVALUE(nEvent,0)
  IF xValue = 2 // RB down
    nCol := JAXPPGETEVENTPARAMETERVALUE(nEvent,2)
    nRow := oHelp:textHeight - JAXPPGETEVENTPARAMETERVALUE(nEvent,3)
    oHelp:editMenu:popup(oHelp:TxDlgArea, {nCol,nRow},, ;
                    XBPMENU_PU_DEFAULT+XBPMENU_PU_MOUSE_RBDOWN)
  ENDIF
ENDIF

RETURN
#endif

* --------------

STATIC FUNCTION RecLock( nWaitTime )

LOCAL nWait

IF DBRLOCK()
  RETURN (.T.)   // locked
ENDIF
DO WHILE .T.
  nWait := nWaitTime
  DO WHILE (nWaitTime=0 .OR. nWait>0)
    IF DBRLOCK()     // locked
      RETURN (.T.)
    ENDIF
    INKEY(.5)     // wait 1/2 second
    nWait -= .5
  ENDDO
ENDDO
RETURN (.F.)      // not locked

/* -------------------- */

STATIC FUNCTION AddRec ( nWaitTime )

LOCAL nWait

APPEND BLANK
IF !NETERR()
   RETURN (.T.)
ENDIF
DO WHILE .T.
  nWait := nWaitTime
  DO WHILE (nWaitTime = 0 .OR. nWait > 0)
    APPEND BLANK
    IF .NOT. NETERR()
      RETURN .T.
    ENDIF
    nWait -= .5
  ENDDO
ENDDO
RETURN .F.

/* --------------------- */

STATIC FUNCTION FiLock( nWaitTime )

LOCAL nWait

IF FLOCK()
  RETURN (.T.)   // locked
ENDIF
DO WHILE .T.
  nWait := nWaitTime
  DO WHILE (nWaitTime=0 .OR. nWait>0)
    INKEY(.5)     // wait 1/2 second
    nWait -= .5
    IF FLOCK()
      RETURN (.T.)    // locked
    ENDIF
  ENDDO
ENDDO
RETURN .F.

* -------------

STATIC FUNCTION _UserId(aApp)

RETURN 'RDONNAY'
// return set_user(USER_USER_KEY)   // returns 7 character user ID key value

* -------------

FUNCTION dc_dbsel ( cAlias )

LOCAL i

cAlias := IIF(Valtype(cAlias)='C',UPPER(cAlias),'')
IF cAlias=='0'
  FOR i := 253 TO 1 STEP -1
    IF Empty(Alias(i))
      SELE (i)
      RETURN .t.
    ENDIF
  NEXT
  RETURN .f.
ENDIF
cAlias := IIF(Valtype(cAlias)='C',cAlias,'')
IF SELECT(cAlias)>0 .AND. !(':'$cAlias)
  SELE SELECT(cAlias)
  RETURN .t.
ENDIF
RETURN .f.



