/*
Test operations of DCPRINT system
*/

#INCLUDE "dcprint.ch"
#INCLUDE "dcdialog.ch"

FUNCTION Main()

LOCAL GetList := {}, cFont := '12.Alaska Crt', lPrinting := .f.

DC_ChDir('..')
SET DEFA TO ..\DATA
SET PATH TO ..\DATA

@ 1,0 DCPUSHBUTTON CAPTION 'Test 1 (HIDE Preview)     ' SIZE 35,2 ;
      TOOLTIP 'Preview a Report of 12 pages (HIDE mode)' ;
      ACTION {||PrintTest1(1,GetList,@lPrinting)} ;
      WHEN {||!lPrinting} ;
      FONT cFont

@ 3,0 DCPUSHBUTTON CAPTION 'Test 2 (Non-HIDE Preview) ' SIZE 35,2 ;
      TOOLTIP 'Preview a Report of 12 pages (Non-HIDE mode)' ;
      ACTION {||PrintTest1(2,GetList,@lPrinting)} ;
      WHEN {||!lPrinting} ;
      FONT cFont

@ 5,0 DCPUSHBUTTON CAPTION 'Test 3 (STOP Preview)     ' SIZE 35,2 ;
      TOOLTIP 'Preview a Report of 12 pages (STOP mode)' ;
      ACTION {||PrintTest1(3,GetList,@lPrinting)} ;
      WHEN {||!lPrinting} ;
      FONT cFont

@ 7,0 DCPUSHBUTTON CAPTION 'Test 4 (NON-STOP Print)   ' SIZE 35,2 ;
      TOOLTIP 'Print a Report of 3 pages to Selected Printer' ;
      ACTION {||PrintTest1(4,GetList,@lPrinting)} ;
      WHEN {||!lPrinting} ;
      FONT cFont

@ 9,0 DCPUSHBUTTON CAPTION 'Test 5 (DEFAULT Print)    ' SIZE 35,2 ;
      TOOLTIP 'Print a Report of 3 pages to DEFAULT Printer' ;
      ACTION {||PrintTest1(5,GetList,@lPrinting)} ;
      WHEN {||!lPrinting} ;
      FONT cFont

@11,0 DCPUSHBUTTON CAPTION 'Test 6 (DEFAULT Preview)  ' SIZE 35,2 ;
      TOOLTIP 'Preview a Report of 3 pages to DEFAULT Printer' ;
      ACTION {||PrintTest1(6,GetList,@lPrinting)} ;
      WHEN {||!lPrinting} ;
      FONT cFont

@13,0 DCPUSHBUTTON CAPTION 'Test 7 (Preview Text File)' SIZE 35,2 ;
      TOOLTIP 'Preview a Text File (README.TXT)' ;
      ACTION {||DC_PrintFile('..\README.TXT',.t.)} ;
      WHEN {||!lPrinting} ;
      FONT cFont

DCREAD GUI FIT ADDBUTTONS SETAPPWINDOW ;
   TITLE 'eXPress++ Print/Preview Test' ;
   EVAL {|o|SetAppWindow(o)}

RETURN nil

* --------------

FUNCTION PrintTest1( nMode, GetList, lPrinting )

   LOCAL oPrinter, i, j, nPage
   LOCAL aForSale := { 'Not for Sale','For Sale','Not Sure' }
   LOCAL cMemo, nLineCount, cMemoLine

   lPrinting := .t.
   DC_GetRefresh(GetList)

   USE COLLECT VIA DBFNTX

   IF nMode = 1

     DCPRINT ON ;
       TO oPrinter ;
       PREVIEW ;
       HIDE ;
       ENABLECANCEL ;
       PSIZE 600,700 ;
       NOSTOP ;
       ZOOMFACTOR 1.0 ;
       ;// SIZE 66, 80

   ELSEIF nMode = 2

     DCPRINT ON ;
       TO oPrinter ;
       ENABLECANCEL ;
       PREVIEW NOSTOP

   ELSEIF nMode = 3

     DCPRINT ON ;
       ENABLECANCEL ;
       TO oPrinter ;
       PREVIEW

   ELSEIF nMode = 4

     DCPRINT ON ;
       TO oPrinter ;
       ENABLECANCEL

   ELSEIF nMode = 5

     DCPRINT ON ;
       TO oPrinter ;
       USEDEFAULT ;
       ENABLECANCEL

   ELSEIF nMode = 6

     DCPRINT ON ;
       TO oPrinter ;
       USEDEFAULT ;
       ENABLECANCEL ;
       PREVIEW

   ENDIF

   IF Valtype(oPrinter) # 'O' .OR. !oPrinter:lActive
      lPrinting := .f.
      DC_GetRefresh(GetList)
      RETURN nil
   ENDIF

   // Page 1

   @ 0,0,65,79 DCPRINT BOX PRINTER oPrinter WHEN {|o|o:lPreview}

   DCPRINT FONT '10.Arial' PRINTER oPrinter

   FOR i := 0 TO 9
      @ i,0 DCPRINT SAY Pad('Line ' + Alltrim(Str(i)),10) + Repl('0123456789',7) ;
         PRINTER oPrinter
   NEXT

   DCPRINT FONT '10.Courier New Bold' PRINTER oPrinter

   DCPRINT EVAL {|o|SaySomeText(o,15,20)} PRINTER oPrinter

   @32,0 DCPRINT SAY 'This text should be cutting the pie' FONT '14.Arial Bold' PRINTER oPrinter

   DCPRINT EVAL _PieBlock(1,.t.,.t.) PRINTER oPrinter
   DCPRINT EVAL _PieBlock(2,.f.,.t.) PRINTER oPrinter
   DCPRINT EVAL _PieBlock(3,.t.,.f.) PRINTER oPrinter

   DCPRINT FONT '8.Tahoma' PRINTER oPrinter

   FOR i := 48 TO 63
      @ i,0 DCPRINT SAY Pad('Line ' + Alltrim(Str(i)),10) + Repl('0123456789',7) PRINTER oPrinter
   NEXT

   DCPRINT EJECT PRINTER oPrinter

   // Page 2 - A line Chart


   DCPRINT EVAL {|o|DrawLineGraph(o,42,10)} PRINTER oPrinter

   DCPRINT EJECT PRINTER oPrinter

   // Page 3 - Different Fonts

   @ 0,0,66,79 DCPRINT BOX PRINTER oPrinter

   DCPRINT FONT '10.Courier New' PRINTER oPrinter
   FOR i := 0 TO 5
      FOR j := 0 TO 6
         @ i,j*11 DCPRINT SAY '0123456789' PRINTER oPrinter FIXED
      NEXT
   NEXT

   DCPRINT FONT '8.Courier New' PRINTER oPrinter
   FOR i := 7 TO 12
      FOR j := 0 TO 6
         @ i,j*11 DCPRINT SAY '0123456789' PRINTER oPrinter FIXED
      NEXT
   NEXT

   DCPRINT FONT '6.Courier New' PRINTER oPrinter
   FOR i := 14 TO 18
      FOR j := 0 TO 6
         @ i,j*11 DCPRINT SAY '0123456789' PRINTER oPrinter FIXED
      NEXT
   NEXT

   DCPRINT FONT '10.Arial' PRINTER oPrinter
   FOR i := 20 TO 24
      FOR j := 0 TO 6
         @ i,j*11 DCPRINT SAY '0123456789' PRINTER oPrinter FIXED
      NEXT
   NEXT

   DCPRINT FONT '8.Tahoma' PRINTER oPrinter
   FOR i := 26 TO 30
      FOR j := 0 TO 6
         @ i,j*11 DCPRINT SAY '0123456789' PRINTER oPrinter FIXED
      NEXT
   NEXT

   // Page 4 - 12

   DO WHILE !Eof() .AND. nMode < 5 .AND. DC_PrinterOk(oPrinter)

      DCPRINT EJECT PRINTER oPrinter

      DCPRINT SIZE 66,80 PRINTER oPrinter

      @ 2,2,20,35 DCPRINT BITMAP COLLECT->bitmap1 PRINTER oPrinter ;
         AUTOSCALE CENTER

      @ 2,37,20,70 DCPRINT BITMAP COLLECT->bitmap2 PRINTER oPrinter ;
         AUTOSCALE CENTER

      DCPRINT FONT '10.Courier New' PRINTER oPrinter

      @ 23, 5, 39, 65 DCPRINT BOX PRINTER oPrinter

      @ 24,7 DCPRINT SAY '    Description: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->descrip   PRINTER oPrinter FONT '10.Courier New Bold'

      @ 25,7 DCPRINT SAY '           Type: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->type      PRINTER oPrinter FONT '10.Courier New Bold'

      @ 26,7 DCPRINT SAY '       Sub-Type: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->sub_type  PRINTER oPrinter FONT '10.Courier New Bold'

      @ 27,7 DCPRINT SAY '      Condition: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->condition PRINTER oPrinter FONT '10.Courier New Bold'

      @ 28,7 DCPRINT SAY '       Location: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->location  PRINTER oPrinter FONT '10.Courier New Bold'

      @ 29,7 DCPRINT SAY '      For Sale?: ' PRINTER oPrinter
      DCPRINT SAY aForSale[COLLECT->for_sale+1]  PRINTER oPrinter ;
         FONT '10.Courier New Bold'

      @ 30,7 DCPRINT SAY '  Original Date: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->date_orig PRINTER oPrinter FONT '10.Courier New Bold'

      @ 31,7 DCPRINT SAY '  Acquired Date: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->date_acqu  PRINTER oPrinter FONT '10.Courier New Bold'

      @ 32,7 DCPRINT SAY ' Original Price: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->orig_price PRINTER oPrinter FONT '10.Courier New Bold'

      @ 33,7 DCPRINT SAY 'Appraised Value: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->appr_value PRINTER oPrinter FONT '10.Courier New Bold'

      @ 34,7 DCPRINT SAY '       Comments: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->comments  PRINTER oPrinter  FONT '10.Courier New Bold'

      @ 35,7 DCPRINT SAY '      Bit Map 1: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->bitmap1   PRINTER oPrinter  FONT '10.Courier New Bold'

      @ 36,7 DCPRINT SAY '      Bit Map 2: ' PRINTER oPrinter
      DCPRINT SAY COLLECT->bitmap2   PRINTER oPrinter  FONT '10.Courier New Bold'

      DCPRINT FONT '16.Arial'  PRINTER oPrinter

      DCPRINT SIZE 40,80 PRINTER oPrinter

      cMemo := Alltrim(COLLECT->memo)
      nLineCount := MLCount(cMemo)
      FOR i := 1 TO nLineCount - 1
         cMemoLine := MemoLine( cMemo, nil, i )
         IF Empty(cMemoLine)
            EXIT
         ENDIF
         @ 25+i,7 DCPRINT SAY cMemoLine PRINTER oPrinter
      NEXT

      SKIP

   ENDDO

   DCPRINT SIZE 66,80
   DCPRINT OFF PRINTER oPrinter
   CLOSE ALL

   lPrinting := .f.
   DC_GetRefresh(GetList)
   Sleep(100)

RETURN nil

* -------------

STATIC FUNCTION _PieBlock(i,lPatterns,lColors)

RETURN {|o|DrawPieChart(o,32,20*i,300,lPatterns,lColors)}

* -------------

FUNCTION SaySomeText( oPrinter, nStart, nEnd )

LOCAL i

FOR i := nStart TO nEnd
  GraStringAt( oPrinter:oPS, oPrinter:PenCoords(i,10), Repl('0123456789',6) )
NEXT

RETURN nil

* -------------

FUNCTION DrawPieChart( oPrinter, nRow, nCol, nRadius, lPatterns, lColors )

LOCAL nX, nY, nStartAngle, aAngle, aPattern, i, aPenCoords, aColors

DEFAULT lPatterns := .t., lColors := .t.

Sleep(.1)
aPenCoords := oPrinter:PenCoords(nRow,nCol)
nX       := aPenCoords[1]        // mid-point of the
nY       := aPenCoords[2]        // pie chart

DEFAULT nRadius  := 150                  // radius

aAngle   := { 30, 100, 15, 75, 80, 60 }
aPattern := { ;                  // various fill attributes
         GRA_SYM_DENSE2, ;
         GRA_SYM_DIAG1 , ;
         GRA_SYM_DENSE4, ;
         GRA_SYM_DIAG2 , ;
         GRA_SYM_DENSE6, ;
         GRA_SYM_DIAG3   }

aColors := { ;                  // various fill colors
         GRA_CLR_RED, ;
         GRA_CLR_BLUE , ;
         GRA_CLR_YELLOW, ;
         GRA_CLR_GREEN , ;
         GRA_CLR_BROWN, ;
         GRA_CLR_PINK }

nStartAngle := 45                // starting angle

FOR i := 1 TO LEN( aAngle )      // draw filled arcs
  DrawFilledPartialArc( {nX, nY}, nRadius, ;
                               nStartAngle, aAngle[i], ;
                               aPattern[i], aColors[i], .T., oPrinter:oPS, lPatterns, lColors )
  nStartAngle += aAngle[i]      // increment starting angle
NEXT

RETURN nil


*****************************************
* Draws filled arcs                     *

*****************************************
FUNCTION DrawFilledPartialArc( aCenter, ;
                               nRadius, ;
                               nStartAngle, ;
                               nSweepAngle, ;
                               nPattern , ;
                               nColor, ;
                               lOutline, ;
                               oPS, ;
                               lPatterns, ;
                               lColors )

LOCAL aAttr, nSegment, aOldAttr := GraSetAttrArea(oPS)

Sleep(.1)
aAttr := Array( GRA_AA_COUNT )   // select fill pattern

IF lPatterns
  aAttr [ GRA_AA_SYMBOL ] := nPattern
ENDIF


IF lColors
  aAttr [ GRA_AA_COLOR ] := nColor
ENDIF

aAttr := GraSetAttrArea( oPS, aAttr )
                                // open segment
nSegment := GraSegOpen( oPS, GRA_SEG_MODIFIABLE )
GraPathBegin( oPS )             // define path
GraPos( oPS, aCenter )          // define arc
GraArc( oPS, aCenter, nRadius,, nStartAngle, nSweepAngle )

GraPathEnd( oPS, .T. )          // end path definition
GraSegClose( oPS )              // close segment

GraSegDraw( oPS, nSegment )     // create path for drawing

GraPathFill( oPS )              // fill path

IF lOutline
  GraSegDraw( oPS, nSegment )   // recreate path
  GraPathOutline( oPS )         // outline path
ENDIF

GraSegDestroy( oPS, nSegment)   // delete segment
GraSetAttrArea( oPS, aOldAttr ) // reset old attributes

RETURN NIL

* -----------------

FUNCTION DrawLineGraph( oPrinter, nRow, nCol )

   LOCAL oPS := oPrinter:oPS, aPenCoords
   LOCAL mTik := "PAR", nMonth, nYear
   LOCAL mName := "Parata Analytics Research"
   LOCAL mTitle := "Price History"
   LOCAL aAttr,i,nX,nY1,y,oFont
   LOCAL hi := -9999, lo := 9999 , nXdivs := 6
   LOCAL ylinc,ylinc1,yinc,xinc,yinc1,nYdivs,yval,xval
   LOCAL nX0   := 50, nY0   := 30               // origins
   LOCAL xlen := 2000, ylen := 1500              // Relative max. values for x and y
   LOCAL xbeg, xend, xAvg
   LOCAL aOldAttr := GraSetAttrString(oPS)

   LOCAL aTMP := ;
      { 59.81, 60.31, 69.94, 80.50, 69.00, 75.75, 66.25, 57.63, 64.38, 62.88, ;
      68.75, 65.56, 66.88, 51.75, 50.13, 50.25, 44.25, 45.13, 48.75, 43.88, ;
      47.13, 33.50, 36.88, 34.00, 31.50, 30.00, 34.50, 34.38, 41.75, 46.50, ;
      44.13, 39.25, 45.25, 43.75, 41.50, 40.50, 38.75, 33.00, 30.63, 29.88, ;
      28.00, 27.25, 24.75, 24.75, 24.13, 24.00, 22.16, 22.25, 20.00, 22.00, ;
      22.50, 23.75, 21.13, 22.00, 21.63, 19.38, 17.88, 16.38, 17.13, 15.63, ;
      15.75, 14.44, 14.19, 12.38, 12.75, 11.75, 11.38, 12.75 }

   LOCAL aData := {}, aBrowData := {}

   Sleep(.1)
   aPenCoords := oPrinter:PenCoords(nRow,nCol)
   nX0        := aPenCoords[1]                  // start of the
   nY0        := aPenCoords[2]                  // line chart

   nYear := 1993
   nMonth := 1
   FOR Y = LEN(aTMP) TO 1 STEP -1
      aAdd(aData,aTmp[y])
      aAdd(aBrowData,{aTmp[y],nMonth,nYear})
      nMonth++
      IF nMonth = 13
         nMonth := 1
         nYear++
      ENDIF
   NEXT Y

   xbeg := get_beg(aData)
   xend := get_end(aData)
   xAvg := get_avg(aData,xbeg,xend)
   nXdivs := 6

   FOR Y = xbeg to xend
      hi := if(aData[Y] > 5*xavg,hi,max(hi,aData[Y]))
      lo := if(aData[Y] > 0,min(lo,aData[Y]),0)
   NEXT Y

   hi := hi * 1.1
   lo := if(lo*.9 < 0, 0, lo*.9)
   Ylinc := (hi-lo)/10
   nYDivs := int((hi-lo)/Ylinc)
   YINC := ylen/(HI-lo)
   XINC := xlen/72

   aAttr := ARRAY( GRA_AM_COUNT )
   aAttr [ GRA_AM_COLOR ] := GRA_CLR_DARKBLUE
   aAttr [ GRA_AM_SYMBOL ] := GRA_MARKSYM_PLUS
   GraSetAttrMarker( oPS, aAttr )

   aAttr := ARRAY( GRA_AL_COUNT )
   aAttr [ GRA_AL_WIDTH ]  := GRA_LINEWIDTH_NORMAL
   aAttr [ GRA_AL_COLOR ] := GRA_CLR_DARKBLUE
   aAttr [ GRA_AL_TYPE ]  := GRA_LINETYPE_SOLID
   GraSetAttrLine( oPS, aAttr )

   aAttr := ARRAY( GRA_AS_COUNT )
   aAttr [ GRA_AS_HORIZALIGN ] := GRA_HALIGN_LEFT
   aAttr [ GRA_AS_VERTALIGN ]  := GRA_VALIGN_HALF
   GraSetAttrString( oPS, aAttr )

   // Draw x and y axis
   GraLine( oPS, {nX0, nY0}, {nX0+xlen, nY0} )
   GraLine( oPS, {nX0, nY0}, {nX0, nY0+ylen} )
   GraLine( oPS, {nX0, nY0+ylen}, {nX0+xlen, nY0+ylen} )
   GraLine( oPS, {nX0+xlen, nY0}, {nX0+xlen, nY0+ylen} )

   // Draw vertical scale
   for y = 0 to nYdivs
      GraMarker  ( oPS, { nX0   , nY0+((ylen/10)*y)} )
      IF lo+(ylinc*nYdivs) >= 1000
         GraStringAt( oPS, { nX0-130, nY0+((ylen/10)*y) }, Str(lo+(ylinc*y),6,0) )
      ELSE
         GraStringAt( oPS, { nX0-130, nY0+((ylen/10)*y) }, Str(lo+(ylinc*y),6,2) )
      ENDIF
   next y

   GraSetAttrString( oPS, aAttr )
   GraStringAt( oPS, { nX0-130, nY0+((ylen/10)*y) }, "Price" )

   aAttr [ GRA_AS_HORIZALIGN ] := GRA_HALIGN_CENTER
   aAttr [ GRA_AS_VERTALIGN ]  := GRA_VALIGN_BOTTOM
   GraSetAttrString( oPS, aAttr )

   GraStringAt( oPS, { nX0 + (xlen/2-20), nY0 + ylen +30 }, "Parata" )

   // Draw horizontal scale
   FOR i := 1 TO nXdivs
      nX := nX0 + ( i*(xlen/nXdivs) )
      GraMarker( oPS, { nX, nY0 } )
      GraStringAt( oPS, { nX-((xlen/nXdivs)/2+10), nY0-50 }, ;
         str(year(date())-6+i))
   NEXT i

   // Draw line
   FOR y := xbeg+1 TO min(xend,72)
      yval := if(aData[y] <= hi, (aData[Y]-lo)*YINC,(hi-lo)*YINC)
      IF yval > 0
         xval := (y*xinc)
         nY1 := nY0+((aData[y-1]-lo)*yinc)
         GraLine( oPS, {nX0+xval-xinc, nY1}, {(nX0+xval), nY0+yval} )
      ENDIF
   NEXT

   aAttr [ GRA_AS_HORIZALIGN ] := GRA_HALIGN_NORMAL
   aAttr [ GRA_AS_VERTALIGN ]  := GRA_HALIGN_NORMAL
   GraSetAttrString( oPS, aAttr )

RETURN NIL


/* ---------------------- */

STATIC FUNCTION GET_END(aArray)

   local xend
   xend := len(aArray)
   do while empty(aArray[XEND]) .and. xend > 1
      xend--
   enddo

return(xend)

/* ----------------------- */

STATIC FUNCTION GET_beg(aArray)

   local xbeg,last
   last := len(aArray)
   xbeg := 1
   do while empty(aArray[Xbeg]) .and. xbeg < last
      xbeg++
   enddo

return(xbeg)

/* --------------------- */

STATIC FUNCTION GET_avg(aArray,xbeg,xend)

   local x
   local xavg := 0
   for x = xbeg to xend
      xavg := xavg + aArray[x]
   next
   xavg := z(xavg,(xend-xbeg+1))

return(xavg)

/* ---------------------- */

STATIC FUNCTION Z(cl_num1,cl_num2)

   LOCAL RET_VAL := 0
   IF cl_num2 <> 0 .and. cl_num2 <> nil
      RET_VAL := (cl_num1/cl_num2)
   ENDIF

return (RET_VAL)




* -----------------

PROC AppSys
RETURN

