/*
This is a Check Printing Form designed for printing with
the DCPRINT system.  All Check data is passed to the
function PrintCheck() in a multi-dimensional array.  This
example is designed for compatiblity with pre-printed
checks made for Quicken/QuickBooks.   See the file
CHECKS.GIF for an example of the check form.
*/

#INCLUDE "dcprint.ch"
#INCLUDE "dcdialog.ch"

#define CHECK_DATE                     1
#define CHECK_PAYTO                    2
#define CHECK_AMOUNT                   3
#define CHECK_MEMO                     4
#define CHECK_NAME                     5
#define CHECK_ADDR1                    6
#define CHECK_ADDR2                    7
#define CHECK_CITY                     8
#define CHECK_STATE                    9
#define CHECK_ZIP                     10
#define CHECK_COUNTRY                 11
#define CHECK_VOUCHER_LINE1           12
#define CHECK_VOUCHER_LINE2           13
#define CHECK_VOUCHER_LINE3           14
#define CHECK_VOUCHER_LINE4           15
#define CHECK_VOUCHER_LINE5           16
#define CHECK_VOUCHER_LINE6           17
#define CHECK_VOUCHER_LINE7           18
#define CHECK_VOUCHER_LINE8           19
#define CHECK_VOUCHER_LINE9           20

#define CHECK_DATE_POS                21
#define CHECK_PAYTO_POS               22
#define CHECK_NAME_POS                23
#define CHECK_AMOUNT_NUM_POS          24
#define CHECK_AMOUNT_WORDS_POS        25
#define CHECK_MEMO_POS                26
#define CHECK_SIGNATURE_POS           27
#define CHECK_VOUCHER1_POS            28
#define CHECK_VOUCHER2_POS            29

#define CHECK_ARRAY_SIZE              29

MEMVAR aCheck

FUNCTION Main()

LOCAL i, lStatus, GetList[0], GetOptions, aData[CHECK_ARRAY_SIZE]

IF File('CHECKS.XPF')

  PRIVATE aCheck
  RESTORE FROM CHECKS.XPF
  aData := M->aCheck

ELSE

  aData[CHECK_DATE]         := Date()
  aData[CHECK_PAYTO]        := Pad('Roger J. Donnay',40)
  aData[CHECK_AMOUNT]       := 0
  aData[CHECK_NAME]         := Pad('Donnay Software Designs',40)
  aData[CHECK_ADDR1]        := Pad('1632 Riverstone Ln #301',40)
  aData[CHECK_ADDR2]        := Space(40)
  aData[CHECK_CITY]         := Pad('Boise',30)
  aData[CHECK_STATE]        := Pad('ID',15)
  aData[CHECK_ZIP]          := Pad('83706',10)
  aData[CHECK_COUNTRY]      := Pad('U.S.A.',20)
  aData[CHECK_MEMO]         := Space(60)

  FOR i := 1 TO 9
    aData[CHECK_VOUCHER_LINE1+i-1] := Space(80)
  NEXT

  aData[CHECK_DATE_POS]         := {  5.0, 65.0 }
  aData[CHECK_PAYTO_POS]        := {  7.5, 10.0 }
  aData[CHECK_NAME_POS]         := { 12.0, 6.0  }
  aData[CHECK_AMOUNT_NUM_POS]   := {  8.0, 65.0 }
  aData[CHECK_AMOUNT_WORDS_POS] := {  9.8, 3.0  }
  aData[CHECK_MEMO_POS]         := { 16.0, 8.0  }
  aData[CHECK_SIGNATURE_POS]    := { 16.0, 50.0 }
  aData[CHECK_VOUCHER1_POS]     := { 24.0, 1.0  }
  aData[CHECK_VOUCHER2_POS]     := { 45.0, 1.0  }

ENDIF

FOR i := 1 TO 9
  aData[CHECK_VOUCHER_LINE1+i-1] := Pad(aData[CHECK_VOUCHER_LINE1+i-1],80)
NEXT

@  1,1 DCSAY 'Date' GET aData[CHECK_DATE] POPUP {|d|DC_PopDate(d)}
@  2,1 DCSAY 'Pay to' GET aData[CHECK_PAYTO]
@  3,1 DCSAY 'Amount' GET aData[CHECK_AMOUNT] PICT '999999.99'
@  4,1 DCSAY 'Memo' GET aData[CHECK_MEMO] GETSIZE 50
@  5,1 DCSAY 'Name' GET aData[CHECK_NAME]
@  6,1 DCSAY 'Address 1' GET aData[CHECK_ADDR1]
@  7,1 DCSAY 'Address 2' GET aData[CHECK_ADDR2]
@  8,1 DCSAY 'City' GET aData[CHECK_CITY]
@  9,1 DCSAY 'State' GET aData[CHECK_STATE]
@ 10,1 DCSAY 'Zip' GET aData[CHECK_ZIP]
@ 11,1 DCSAY 'Country' GET aData[CHECK_COUNTRY]

@  5,1 DCSAY 'Addressee:' COLOR GRA_CLR_BLUE SAYLEFT SAYSIZE 0

@ 13,1 DCSAY 'Line 1' GET aData[CHECK_VOUCHER_LINE1]
@ 14,1 DCSAY 'Line 2' GET aData[CHECK_VOUCHER_LINE2]
@ 15,1 DCSAY 'Line 3' GET aData[CHECK_VOUCHER_LINE3]
@ 16,1 DCSAY 'Line 4' GET aData[CHECK_VOUCHER_LINE4]
@ 17,1 DCSAY 'Line 5' GET aData[CHECK_VOUCHER_LINE5]
@ 18,1 DCSAY 'Line 6' GET aData[CHECK_VOUCHER_LINE6]
@ 19,1 DCSAY 'Line 7' GET aData[CHECK_VOUCHER_LINE7]
@ 20,1 DCSAY 'Line 8' GET aData[CHECK_VOUCHER_LINE8]
@ 21,1 DCSAY 'Line 9' GET aData[CHECK_VOUCHER_LINE9]

@ 13,1 DCSAY 'Voucher:' COLOR GRA_CLR_BLUE SAYLEFT SAYSIZE 0

@ 1,85 DCSAY 'Print Coordinates:' COLOR GRA_CLR_BLUE SAYSIZE 0
@ 2,85 DCSAY 'Row' COLOR GRA_CLR_BLUE SAYSIZE 0
@ 2,91 DCSAY 'Col' COLOR GRA_CLR_BLUE SAYSIZE 0

@ 3,70 DCSAY 'Date'
@ 3,85 DCGET aData[CHECK_DATE_POS,1] PICT '99.9'
@ 3,91 DCGET aData[CHECK_DATE_POS,2] PICT '99.9'

@ 4,70 DCSAY 'Pay To'
@ 4,85 DCGET aData[CHECK_PAYTO_POS,1] PICT '99.9'
@ 4,91 DCGET aData[CHECK_PAYTO_POS,2] PICT '99.9'

@ 5,70 DCSAY 'Name/Address'
@ 5,85 DCGET aData[CHECK_NAME_POS,1] PICT '99.9'
@ 5,91 DCGET aData[CHECK_NAME_POS,2] PICT '99.9'

@ 6,70 DCSAY 'Amount (Num)'
@ 6,85 DCGET aData[CHECK_AMOUNT_NUM_POS,1] PICT '99.9'
@ 6,91 DCGET aData[CHECK_AMOUNT_NUM_POS,2] PICT '99.9'

@ 7,70 DCSAY 'Amount (Words)'
@ 7,85 DCGET aData[CHECK_AMOUNT_WORDS_POS,1] PICT '99.9'
@ 7,91 DCGET aData[CHECK_AMOUNT_WORDS_POS,2] PICT '99.9'

@ 8,70 DCSAY 'Memo'
@ 8,85 DCGET aData[CHECK_MEMO_POS,1] PICT '99.9'
@ 8,91 DCGET aData[CHECK_MEMO_POS,2] PICT '99.9'

@ 9,70 DCSAY 'Signature'
@ 9,85 DCGET aData[CHECK_SIGNATURE_POS,1] PICT '99.9'
@ 9,91 DCGET aData[CHECK_SIGNATURE_POS,2] PICT '99.9'

@10,70 DCSAY 'Voucher 1'
@10,85 DCGET aData[CHECK_VOUCHER1_POS,1] PICT '99.9'
@10,91 DCGET aData[CHECK_VOUCHER1_POS,2] PICT '99.9'

@11,70 DCSAY 'Voucher 2'
@11,85 DCGET aData[CHECK_VOUCHER2_POS,1] PICT '99.9'
@11,91 DCGET aData[CHECK_VOUCHER2_POS,2] PICT '99.9'

@ 23,1 DCPUSHBUTTON CAPTION 'Exit' SIZE 9,1.2 ;
       ACTION {||DC_ReadGuiEvent(DCGUI_EXIT_OK,GetList)}

@ 23,12 DCPUSHBUTTON CAPTION 'Print' SIZE 9,1.2 ;
       ACTION {||PrintCheck(aData)}

DCGETOPTIONS SAYWIDTH 100 SAYRIGHT

DCREAD GUI FIT OPTIONS GetOptions ;
   TITLE 'Create and Print a Check' ;
   TO lStatus SETAPPWINDOW

IF lStatus
  aCheck := aData
  SAVE ALL LIKE aCheck TO CHECKS.XPF
ENDIF

RETURN nil

* -------------------

STATIC FUNCTION PrintCheck( aData )

LOCAL oPrinter, nOffset, i

DCPRINT ON PREVIEW TO oPrinter HIDE NONSTOP

DCPRINT FONT '12.Arial'

// Print Date

@ aData[CHECK_DATE_POS,1], aData[CHECK_DATE_POS,2] ;
  DCPRINT SAY DtoC(aData[CHECK_DATE])

// Print PayTo

@ aData[CHECK_PAYTO_POS,1], aData[CHECK_PAYTO_POS,2] ;
  DCPRINT SAY Trim(aData[CHECK_PAYTO])

// Print Amount (in Words)

@ aData[CHECK_AMOUNT_WORDS_POS,1], aData[CHECK_AMOUNT_WORDS_POS,2] ;
  DCPRINT SAY NumToWords(aData[CHECK_AMOUNT])

// Print Memo

@ aData[CHECK_MEMO_POS,1], aData[CHECK_MEMO_POS,2] ;
  DCPRINT SAY Trim(aData[CHECK_MEMO])

// Print Amount (numeric)

@ aData[CHECK_AMOUNT_NUM_POS,1], aData[CHECK_AMOUNT_NUM_POS,2] ;
  DCPRINT SAY aData[CHECK_AMOUNT] PICT '999999.99' FONT '14.Courier New Bold'

// Print Address Lines

DCPRINT FONT '11.Courier New'

@ aData[CHECK_NAME_POS,1], aData[CHECK_NAME_POS,2] ;
  DCPRINT SAY Trim(aData[CHECK_NAME])

@ aData[CHECK_NAME_POS,1]+1, aData[CHECK_NAME_POS,2] ;
  DCPRINT SAY Trim(aData[CHECK_ADDR1])

nOffset := 0
IF !Empty(aData[CHECK_ADDR2])
  @ aData[CHECK_NAME_POS,1]+2, aData[CHECK_NAME_POS,2] ;
    DCPRINT SAY Trim(aData[CHECK_ADDR2])
  nOffset := 1
ENDIF

@ aData[CHECK_NAME_POS,1]+2+nOffset, aData[CHECK_NAME_POS,2] ;
  DCPRINT SAY Trim(aData[CHECK_CITY]) + ', ' + ;
  Trim(aData[CHECK_STATE]) + ' ' + Trim(aData[CHECK_COUNTRY])

// Print Vouchers

@ aData[CHECK_VOUCHER1_POS,1], aData[CHECK_DATE_POS,2] ;
  DCPRINT SAY DtoC(aData[CHECK_DATE]) FONT '12.Arial'

DCPRINT FONT '11.Lucida Console'
FOR i := 1 TO 9
  @ aData[CHECK_VOUCHER1_POS,1]+i+1, aData[CHECK_VOUCHER1_POS,2] ;
    DCPRINT SAY Strtran(Trim(aData[CHECK_VOUCHER_LINE1+i-1]),' ',Chr(255))
NEXT

@ aData[CHECK_VOUCHER2_POS,1], aData[CHECK_DATE_POS,2] ;
  DCPRINT SAY DtoC(aData[CHECK_DATE]) FONT '12.Arial'

FOR i := 1 TO 9
  @ aData[CHECK_VOUCHER2_POS,1]+i+1, aData[CHECK_VOUCHER2_POS,2] ;
    DCPRINT SAY Strtran(Trim(aData[CHECK_VOUCHER_LINE1+i-1]),' ',Chr(255))
NEXT

DCPRINT OFF

RETURN nil

* -------------------

FUNCTION NumToWords( nAmount )

LOCAL cResult, cCents, cAmount := Str(nAmount)

cCents := Substr(cAmount,At('.',cAmount)+1)
cResult = _NumToWords( nAmount )
cResult += ' and ' + cCents + '/100'

RETURN cResult

* -------------------

FUNCTION _NumToWords( nAmount )

LOCAL nTemp, cResult := " ", nQualNo
LOCAL nDiv := 10 ^ ( Int( sol10(nAmount) / 3 ) * 3 )
LOCAL ones  := { "",     " One",   " Two",   " Three", " Four", " Five",  ;
                 " Six", " Seven", " Eight", " Nine"                      ;
               }

LOCAL teens := { " Ten",      " Eleven",    " Twelve",   ;
                 " Thirteen", " Fourteen",  " Fifteen",  ;
                 " Sixteen",  " Seventeen", " Eighteen", ;
                 " Nineteen"                             ;
               }

LOCAL tens  :=  { "", "", " Twenty", " Thirty", " Forty", " Fifty", ;
                  " Sixty", " Seventy", " Eighty", " Ninety"  }

LOCAL qualifiers := { "", " Thousand", " Million", " Billion", " Trillion" }

nTemp   := int(nAmount % nDiv)
nAmount := int(nAmount / nDiv)
nQualNo := int( sol10( nDiv ) / 3 ) + 1
cResult += grp_to_words(nAmount, qualifiers[ nQualNo ], ones, teens, tens )

IF nTemp > (nDiv /= 1000) .and. (nDiv > 1)
  cResult += _NumToWords( nTemp, nDiv )
ELSE
  cResult += grp_to_words(nTemp, "", ones, teens, tens)
ENDIF

RETURN LTrim(cResult)

* -----------

STATIC FUNCTION grp_to_words( nGrp, sQual, ones, teens, tens )

LOCAL cResult := "", nTemp

nTemp   := int(nGrp % 100)
nGrp    := int(nGrp / 100)

cResult += ones[ nGrp + 1 ] + iif( nGrp > 0, " Hundred", "")

DO CASE
  CASE nTemp > 19
    cResult += tens[ int( nTemp / 10 ) + 1 ]
    cResult += ones[ int( nTemp % 10 ) + 1 ]
  CASE nTemp < 20 .and. nTemp > 9
    cResult += teens[ int( nTemp % 10 ) + 1 ]
  CASE nTemp < 10 .and. nTemp > 0
    cResult += ones[ int( nTemp) + 1 ]
ENDCASE

RETURN cResult + sQual

* ----------

STATIC FUNCTION sol10( nNumber )

LOCAL sTemp := LTrim( Str( Int(nNumber) ) )

RETURN len(sTemp) - 1

* ------------

PROC appsys
RETURN


