// Using the Internet Explorer Universal Browser OCX in an Xbase++ form
//
// The sample below shows how to:
//   - create an instance of the IE WebBrowser OCX in an Xbase++ form
//   - turn that Xbase++ form into a full featured Web browser, with navigation
//     capabilities
//

/*
This is an Xbase++ application that is used to poll websites and URLs
based on information in WEBPOLL.URL:

WEBPOLL.URL contains lines of text informing the web browser which URL
to run and how long to wait to go to the next URL.  It was originally
written for a customer who needed to display images on a large screen
during an auction.  A seperately running Clipper program, controlled by
the auctioneer, would write out a list of .JPG files to WEBPOLL.URL and
the Xbase++ program would automatically display the images in a
Web Browser.
*/

#include "Gra.ch"
#include "dcdialog.ch"
#include "appevent.ch"

#define JARESIZESTYLE_X1LEFT        0x0
#define JARESIZESTYLE_X1RIGHT       0x1
#define JARESIZESTYLE_X2LEFT        0x00
#define JARESIZESTYLE_X2RIGHT       0x10
#define JARESIZESTYLE_Y1TOP         0x000
#define JARESIZESTYLE_Y1BOTTOM      0x100
#define JARESIZESTYLE_Y2TOP         0x0000
#define JARESIZESTYLE_Y2BOTTOM      0x1000

#define JAHANDLE_PUNK      0
#define JAHANDLE_HWND		2

#define _WebBrowser_
#include "\JACE30\SAMPLES\Xbase++\SHDOCVW.ch"
#include "\JACE30\SAMPLES\Xbase++\SHDOCVW.prg"

PROCEDURE Main

WebPoll()

RETURN

* -------------

PROCEDURE AppSys
RETURN

* -------------

PROCEDURE WebPoll

LOCAL GetList := {}, cURL := Pad('www.donnay-software.com',100), ;
      oWebBrowser, oStatic, oStatus, oURL, oThread, oDlg, oDlgArea, ;
      GetOptions, nTimer := 1, cUrlPath := Space(100), lPolling := .f.

// REQUIRED: a separate thread polling and dispatching the various ActiveX events.
oThread := Thread():new()
oThread:start( "JAProcessOCXEvents")

@  0, 0 DCSTATIC TYPE XBPSTATIC_TYPE_RECESSEDBOX SIZE 90,20.5 OBJECT oStatic

@ 21, 0 DCSTATIC TYPE XBPSTATIC_TYPE_TEXT SIZE 90,1 ;
        FONT '10.Arial' COLOR GRA_CLR_BLUE OBJECT oStatus

@  0, 0 DCDIALOG oDlg NOTITLEBAR DRAWINGAREA oDlgArea SIZE 90,20.5 PARENT oStatic

@ 22, 0 DCPUSHBUTTON CAPTION "Navigate" SIZE 10,1.2  ;
        ACTION {|| oWebBrowser:Navigate(oURL:getData(),0) }

@ 22,13 DCGET cURL GETSIZE 35 GETOBJECT oURL

@ 22,50 DCPUSHBUTTON CAPTION 'Back' SIZE 9,1.2  ACTION {||oWebBrowser:GoBack()}

@ 22,60 DCPUSHBUTTON CAPTION 'Forward' SIZE 9,1.2  ACTION {||oWebBrowser:GoForward()}

@ 22,70 DCPUSHBUTTON CAPTION 'Home' SIZE 9,1.2  ACTION {||oWebBrowser:GoSearch()}

@ 22,80 DCPUSHBUTTON CAPTION {||IIF( lPolling,'Cancel','Poll')} SIZE 9,1.2 ;
        ACTION {||Setup(@nTimer,@cUrlPath,@lPolling), ;
                  SetTimerEvent(nTimer * 100, ;
                       {||Poll(@nTimer,@cUrlPath,oWebBrowser,@lPolling)}), ;
                  DC_GetRefresh(GetList) }

DCGETOPTIONS AUTORESIZE

DCREAD GUI FIT TITLE 'Universal Web Browser' ;
   OPTIONS GetOptions ;
   EVAL {|o| CreateWebBrowser( o:drawingArea, oDlgArea, oStatus, @oWebBrowser )}

oWebBrowser:Disconnect()

RETURN

* ------------------

STATIC FUNCTION CreateWebBrowser( oWindow, oDlgArea, oStatus, oWebBrowser )

LOCAL nWidth, nHeight, nCol, nRow, punk

// Create an ActiveX instance in the Form client area, and wrap it in an Xbase++ class instance
oWebBrowser:= WebBrowser():new()
nWidth  := oDlgArea:currentSize()[1]
nHeight := oDlgArea:currentSize()[2]
nCol    := oDlgArea:currentPos()[1]
nRow    := oDlgArea:currentPos()[2]

punk := JAXPPCREATECONTROL(oDlgArea:getHWND(),;
         "WebBrowser",nCol,nRow,nWidth,nHeight,"ProgID:Shell.Explorer.2",1)

JAXPPSETCONTROLRESIZESTYLE(punk,JAHANDLE_PUNK,JARESIZESTYLE_X2RIGHT + ;
       JARESIZESTYLE_Y2BOTTOM)

oWebBrowser:Connect(punk)

// Connect to the events fired by the ActiveX.
oWebBrowser:SubscribeToEvents({|hEvent| ReflectEventToStatusText(hEvent,oStatus) })

RETURN nil


*/

// Function called by the Event codeblock to fully analyze the received Event,
// The code below also shows how to analyze the received event and take any
// appropriate action, given its name.
PROCEDURE ReflectEventToStatusText(hEvent,oSTStatusText)
LOCAL eventname
LOCAL status

 eventname=JAXPPGETEVENTNAME(hEvent)
 IF eventname="StatusTextChange"
   // Obtain the value of the 'Text' parameter
   status=JAXPPGETEVENTPARAMETERVALUE(hEvent,"Text")
   // Reflect the parameter to the StatusText label control
   oSTStatusText:setCaption(status)
 END

RETURN


// REQUIRED by the JA runtime to process smoothly the OCX events sent by the various ActiveX instances.
PROCEDURE JAProcessOCXEvents()
  JAXPPPROCESSEVENTS()
RETURN

 * ---------------------

STATIC FUNCTION SetUp( nTimer, cUrlPath, lPolling )

LOCAL GetList := {}, GetOptions, lOk

DEFAULT nTimer := 0, ;
        cUrlPath := Space(100)

IF lPolling

  lPolling := .f.
  nTimer := 0

ELSE

  @ 1,1 DCSAY 'Time between Polls (seconds)' GET nTimer PICT '999' ;
        SAYSIZE 30 SAYRIGHT

  @ 3,1 DCSAY 'Directory to look for WEBPOLL.URL' GET cUrlPath ;
        POPUP {|c|DC_PopFile(c)} GETSIZE 40 SAYSIZE 30 SAYRIGHT

  DCREAD GUI FIT ADDBUTTONS TO lOk ;
    ENTEREXIT ;
    TITLE 'Setup Polling Parameters'

  IF !lOk
    nTimer := 0
    lPolling := .f.
  ELSE
    lPolling := .t.
  ENDIF

ENDIF

RETURN nil

* ------------------------

STATIC FUNCTION Poll( nTimer, cUrlPath, oWebBrowser, lPolling )

LOCAL nHandle, cNewUrl, nWait := 0, cTokens, aTokens, i, aUrl := {}
STATIC cUrl := '', lInPolling := .f., lOk := .t.

SetTimerEvent( 0 )
IF !lPolling
  RETURN nil
ENDIF

nHandle := DC_TxtOpen(cUrlPath + '\WEBPOLL.URL')
IF nHandle <= 0
  nHandle := DC_TxtOpen('WEBPOLL.URL')
ENDIF

IF nHandle > 0
  DC_AppEvent(@lOk,0,.01)
  DO WHILE !DC_TxtEof(nHandle) .AND. lPolling
    cTokens := DC_TxtLine( nHandle )
    IF Empty(cTokens)
      EXIT
    ENDIF
    aTokens := DC_TokenArray( cTokens, ',' )
    AADD( aUrl, aTokens )
    DC_TxtSkip( nHandle, 1 )
  ENDDO
  DC_TxtClose(nHandle)
  cNewUrl := aUrl[1,1]
  IF Len(aUrl) > 1
    FOR i := 1 TO Len(aUrl)
      IF !lPolling
        EXIT
      ENDIF
      DC_AppEvent(@lOk,0,.01)
      DCQOUT aUrl[i,1], Alltrim(Str(Val(aUrl[i,2]))) + ' seconds'
      oWebBrowser:Navigate(aUrl[i,1],0)
      nWait := Val(aUrl[i,2])
      IF nWait > 0
        Sleep( nWait * 100 )
      ENDIF
    NEXT
  ELSEIF !cUrl == cNewUrl
    cUrl := cNewUrl
    DCQOUT cUrl
    oWebBrowser:Navigate(cUrl,0)
  ENDIF

  IF lPolling
    SetTimerEvent(nTimer * 100, ;
         {||Poll(@nTimer,@cUrlPath,oWebBrowser,@lPolling)})
  ENDIF

ENDIF

RETURN nil



