/*
 ķ
   Program..: XPPLOAD.PRG                                  
   Author...:  Roger J. Donnay                             
   Notice...: (c) DONNAY Software Designs 1987-2000        
   Date.....: July 23, 2000                                
   Notes....: Xbase++ Program Loader                       
 Ľ

XPPLOAD.EXE is use to launch large Xbase++ programs and display
a "Loading <program>"  dialog window while all the application
.DLLs are loading.  Large Xbase++ programs can take a very long
time to load, especially on a Network or when the workstation
has a small amount of cache memory available.

XPPLOAD.EXE writes to a file named XPPLOAD.TXT.

The "Loading <program>"  dialog window will stay on the screen
until the XPPLOAD.TXT file is erased.

Place the following line of code in your application just before
calling the main dialog window:

FErase('XPPLOAD.TXT')

XPPLOAD.EXE will keep monitoring XPPLOAD.TXT.  As soon as the
file no longer exists, or the name of the program being run is
removed from the file, the "Loading <program>"  dialog will be
destroyed and XPPLOAD.EXE will quit.  In the event that the
called program never erases XPPLOAD.TXT, XPPLOAD.EXE will
terminate after 30 seconds.

XPPLOAD.EXE requires only a minimum of Xbase++ .DLLs so it should
load very fast and display the "Loading <program>"  dialog
immediately.
*/

#INCLUDE "fileio.ch"
#INCLUDE "gra.ch"
#INCLUDE "inkey.ch"
#INCLUDE "xbp.ch"

PROCEDURE Main( cProgramToRun, p1, p2, p3, p4, p5, p6, p7, p8, p9 )

LOCAL aPrograms := {}, cProgram, nSeconds := Seconds(), oScrn, ;
      nHandle, i, lStillLoading := .t.

IF Empty(cProgramToRun)
  MsgBox( 'Must provide name of Program to Run' )
  QUIT
ENDIF

cProgramToRun := Alltrim(Upper(cProgramToRun))

IF !Right( cProgramToRun, 4 ) == '.EXE'
  cProgramToRun += '.EXE'
ENDIF

p1 := IIF( Valtype(p1)='C',p1,'' )
p2 := IIF( Valtype(p2)='C',p2,'' )
p3 := IIF( Valtype(p3)='C',p3,'' )
p4 := IIF( Valtype(p4)='C',p4,'' )
p5 := IIF( Valtype(p5)='C',p5,'' )
p6 := IIF( Valtype(p6)='C',p6,'' )
p7 := IIF( Valtype(p7)='C',p7,'' )
p8 := IIF( Valtype(p8)='C',p8,'' )
p9 := IIF( Valtype(p9)='C',p9,'' )

oScrn := _WaitOn('Loading ' + cProgramToRun)

RunShell( '/C START ' + cProgramToRun + ' ' + p1 + ' ' + p2 + ' ' + ;
                        p3 + ' ' + p4 + ' ' + p5 + ' ' + p6 + ' ' + ;
                        p7 + ' ' + p8 + ' ' + p9 , , .t., .f. )

nHandle := _TxtOpen('XPPLOAD.TXT')

BEGIN SEQUENCE

IF nHandle >= 0
  DO WHILE !_TxtEof(nHandle)
    cProgram := Alltrim(Upper(_TxtLine(nHandle)))
    IF Empty(cProgram)
      EXIT
    ELSEIF cProgram == cProgramToRun
      BREAK
    ENDIF
    AAdd(aPrograms,Alltrim(Upper(cProgram)))
    _TxtSkip(nHandle,1)
  ENDDO
  FClose(nHandle)
ENDIF
AAdd(aPrograms,cProgramToRun)
nHandle := FCreate('XPPLOAD.TXT')
FOR i := 1 TO Len(aPrograms)
  FWrite(nHandle,aPrograms[i]+Chr(13)+Chr(10))
NEXT

END SEQUENCE
FClose(nHandle)

DO WHILE lStillLoading .AND. Inkey() # K_ESC .AND. Seconds() - nSeconds < 30
  Sleep(100)
  nHandle := _TxtOpen('XPPLOAD.TXT')
  lStillLoading := .f.
  IF nHandle >= 0
    DO WHILE !_TxtEof(nHandle)
      cProgram := _TxtLine(nHandle)
      IF cProgram == cProgramToRun
        lStillLoading := .t.
        EXIT
      ENDIF
      _TxtSkip(nHandle,1)
    ENDDO
    FClose(nHandle)
  ENDIF
ENDDO

_Impl(oScrn)

RETURN

* ------------

FUNCTION _txtopen ( cFileName, nMode )

LOCAL nHandle := FOpen(cFileName, nMode)

RETURN nHandle

* -----------------

FUNCTION _txteof ( nHandle )

LOCAL nEnd,nCurrent

nCurrent := FSeek(nHandle,0,1)
nEnd := FSeek(nHandle,0,2) - 1
FSeek(nHandle,nCurrent,0)
RETURN nCurrent>=nEnd

* -----------------

FUNCTION _txtskip ( nHandle, nLines )

LOCAL cString, cChar, nPos, nEnd, i, cEol

cEol := Chr(13)+Chr(10)

FOR i := 1 TO nLines
  nPos := FSeek( nHandle, 0, 1 ) // Save current pointer
  cString := Space(4000)
  FRead( nHandle, @cString, 4000 )
  nEnd := At( cEol, cString ) // End of line
  IF nEnd=0
    EXIT
  ENDIF
  FSeek( nHandle, nPos, 0 ) // Go back to saved position
  nPos := FSeek( nHandle, nPos+nEnd, 0 ) // Go to start of next line
NEXT
RETURN nil

/* ----------------- */

FUNCTION _txtline ( nHandle )

LOCAL cString, cChar, nPos, nEnd, i, cEol

cEol := Chr(13)+Chr(10)

nPos := FSeek( nHandle, 0, 1 )
cString := FReadStr( nHandle, 1000 )

nEnd := At( cEol, cString )
FSeek( nHandle, nPos, 0 )
IF nEnd=0
  RETURN cString
ENDIF
cString := SubStr( cString, 1, nEnd-1)
FOR i := 1 TO Len(cEol)
  cString := StrTran( cString,SubStr(cEol,i,1),'')
NEXT
RETURN cString

* -------------

FUNCTION _waiton ( aMessage )

LOCAL nWidth, i, lPrint, oThread, cDevice, aSaveScrn, ;
      nCol, nRow, oDlg, drawingArea, oXbp, nHeight, oParent, ;
      lWorking := .t.

IF Valtype(aMessage)='U'
  aMessage := ' Please wait... '
ENDIF
IF Valtype(aMessage)='C'
  aMessage := { aMessage }
ENDIF
nWidth := 0
FOR i := 1 TO LEN(aMessage)
  nWidth := Max(nWidth,LEN(aMessage[i]))
NEXT

oParent := AppDeskTop()
nWidth := Max((nWidth*8)+20,200)
nHeight := LEN(aMessage)*17 + 50 + IIF( lWorking, 10, 0 )
nCol := (oParent:currentSize()[1]-nWidth)/2
nRow := (oParent:currentSize()[2]-nHeight)/2
oDlg := XbpDialog():new(oParent,oParent,{nCol,nRow},{nWidth,nHeight},,.f.)
oDlg:visible := .F.
oDlg:tasklist := .f.
oDlg:title := AppName()
oDlg:create()
drawingArea := oDlg:drawingArea
drawingArea:setColorBG( GRA_CLR_PALEGRAY   )
drawingArea:setFontCompoundName( "10.Helv" )
FOR i := 1 TO ( LEN(aMessage) + IIF( lWorking,1,0 ) )
  oXbp := XbpStatic():new( drawingArea, , ;
           { 20, nHeight - ( 30+(i*20)) }, {Max(nWidth*7,200),20} )
  IF i <= Len(aMessage)
    oXbp:caption := aMessage[i]
  ELSE
    oXbp:caption := ''
  ENDIF
  oXbp:create()
NEXT
oDlg:show()
oDlg:close := {||_Impl(aSaveScrn)}
IF lWorking
   oThread := Thread():new()
   oDlg:cargo := {oXbp,.t.}
   oThread:Start({||_Working(@oDlg)})
ENDIF
aSaveScrn := { , , , , , , , , , oDlg, SetAppWindow(), nil, .t., oThread }

RETURN aSaveScrn

/* --------------- */

STATIC FUNCTION _Working( oDlg )

LOCAL nCount := 1
DO WHILE Valtype(oDlg) = 'O' .AND. oDlg:cargo[2] .AND. oDlg:status() > 0
  SetAppFocus(oDlg)
// oDlg:setModalState(XBP_DISP_APPMODAL)
  Sleep(20)
  IF oDlg:status() > 0
    oDlg:cargo[1]:SetCaption(Repl(Chr(31),nCount))
  ENDIF
  nCount++
  IF nCount > 25
    nCount := 0
  ENDIF
ENDDO

RETURN nil

* -------------

FUNCTION _impl( acSaveScreen )

IF Valtype(acSaveScreen[10]:cargo) = 'A' .AND. Len(acSaveScreen[10]:cargo) >= 2
  acSaveScreen[10]:cargo[2] := .f.
  DO WHILE Valtype(acSaveScreen[14]) = 'O' .AND. acSaveScreen[14]:active
    Sleep(1)
  ENDDO
ENDIF
IF acSaveScreen[10]:status() > 0
  acSaveScreen[10]:Destroy()
ENDIF

RETURN .t.

* -------------

PROCEDURE AppSys()
RETURN

* -------------

PROCEDURE DbeSys()
RETURN
